import unittest

from ..es_builder import ESBuilder
from ..es_query_util import ESQueryUtil
from ..in_query_param import InQueryParam
from ..base import __EXAMPLE_DOC_TYPE as EXAMPLE_DOC_TYPE


class TestESBuilder(unittest.TestCase):

    example_builder = ESBuilder(EXAMPLE_DOC_TYPE)

    def test_properties(self):
        self.assertRaises(
            AttributeError,
            setattr,
            self.example_builder,
            'main_dsl',
            {}
        )

        self.assertRaises(
            AttributeError,
            setattr,
            self.example_builder,
            'page_dsl',
            {}
        )

    def test_get_dsl(self):

        example_util = ESQueryUtil(EXAMPLE_DOC_TYPE)
        in_param = InQueryParam(in_param_key='all_number', in_param_value='18512341234')
        es_param = example_util.get_es_query_param(in_param)
        page_param = example_util.get_page_param_group(page=2, page_size=10)

        self.example_builder.addQueryGroup(es_param).addQueryGroup(page_param)

        expected_main_dsl = {
            'query': {
                'bool': {
                    'should': [
                        {
                            'term': {'number': '18512341234'},
                        },
                        {
                            'nested': {
                                'path': 'inner',
                                'query': {
                                    'bool': {
                                        'filter': {
                                            'term': {
                                                'inner.number': '18512341234'
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    ]
                }
            }
        }

        self.assertDictEqual(self.example_builder.main_dsl, expected_main_dsl, msg="Test ESBuilder.main_dsl Failed")

        expected_page_dls = {
            'from': 10,
            'size': 10
        }
        expected_page_dls.update(expected_main_dsl)

        self.assertDictEqual(self.example_builder.page_dsl, expected_page_dls, msg="Test ESBuilder.page_dsl Failed")


if __name__ == '__main__':
    unittest.main()



from .constants import DocType
from .es_util.constants import ESParamOp


# 存放各文档的查询 mapping
QUERY_MAPPING = {
    # 订单数据
    DocType.ORDER_V2: {
        'phone': [
            {
                'path': 'user.user_name',
            },
            {
                'path': 'shipping_address.phone'
            }
        ],
        'name': [
            {
                'path': 'user.name',
            },
            {
                'path': 'shipping_address.name'
            }
        ],
        'shipping_phone': {
            'path': 'shipping_address.phone'
        },
        'order_num': {
            'path': 'order_number'
        },
        'distribute_id': {
            'path': 'distributes.id'
        },
        'distribute_number': {
            'path': 'distributes.number'
        },
        'distribute_status': {
            'path': 'distributes.status'
        },
        'brand_id': {
            'path': 'lines.brand_id'
        },
        'shop_num': {
            'path': 'lines.shop_num',
        },
        'shop_id': {
            'path': 'lines.shop_id'
        },
        'product_id': {
            'path': 'lines.product_id'
        },
        'product_title': {
            'path': 'lines.title'
        },
        'hg_code': {
            'path': 'lines.hg_code'
        },
        'created_time_after': {
            'path': 'created_time',
            'op': ESParamOp.gte,
        },
        'created_time_before': {
            'path': 'created_time',
            'op': ESParamOp.lte,
        }
    },
    DocType.ORDER_V3: {
        'phone': [
            {
                'path': 'user.user_name',
            },
            {
                'path': 'shipping_address.phone'
            }
        ],
        'name': [
            {
                'path': 'user.name',
            },
            {
                'path': 'shipping_address.name'
            }
        ],
        'shipping_phone': {
            'path': 'shipping_address.phone'
        },
        'order_num': {
            'path': 'number'
        },
        'distribute_id': {
            'path': 'distributes.id'
        },
        'distribute_number': {
            'path': 'distributes.number'
        },
        'distribute_status': {
            'path': 'distributes.status'
        },
        'brand_id': {
            'path': 'lines.brand_id'
        },
        'shop_num': {
            'path': 'lines.shop_num',
        },
        'shop_id': {
            'path': 'lines.shop_id'
        },
        'product_id': {
            'path': 'lines.product_id'
        },
        'product_title': {
            'path': 'lines.title'
        },
        'hg_code': {
            'path': 'lines.hg_code'
        },
        'created_time_after': {
            'path': 'created_time',
            'op': ESParamOp.gte,
        },
        'created_time_before': {
            'path': 'created_time',
            'op': ESParamOp.lte,
        }
    }
}


QUERY_MAPPING_SPECIAL_FUNC = {
    # 查询的特殊函数
}



import warnings

from django.db import models


from ..core.constants import PaymentPayer
from ..core.base_models import BaseModelNew
from . import APP_LABEL


class Coupon(BaseModelNew):
    '''
    优惠券
    '''

    def __init__(self, *args, **kwargs):
        warnings.warn("Coupon 须解耦, 请使用服务间调用请求积分数据", DeprecationWarning)
        super().__init__(self, *args, **kwargs)

    def __new__(cls, *args, **kwargs):
        warnings.warn("Coupon 须解耦, 请使用服务间调用请求积分数据", DeprecationWarning)
        return super().__new__(cls, *args, **kwargs)

    title = models.CharField(
        "优惠券名字", max_length=100, blank=True, null=True)
    # description会给用户展示
    description = models.CharField(
        "优惠券描述", max_length=500, blank=True, null=True)
    discount_amount = models.PositiveIntegerField(
        '折扣金额',
        blank=True, null=True)
    # image = models.ImageField("logo", upload_to=settings.SPARROW_COUPON, blank=True, null=True)
    image_id = models.PositiveIntegerField("优惠券图片", blank=True, null=True)
    # 条件
    c_payment = models.DecimalField(
        "金额条件",
        decimal_places=2, max_digits=12,
        default=0.00)
    # 库存
    count = models.PositiveIntegerField('优惠券总数', default=0)
    # 费用承担方
    payer = models.CharField(
        "费用承担方", max_length=10,
        choices=PaymentPayer.PAYER_CHOICES,
        default=PaymentPayer.HG)
    # 有效期
    start_time = models.DateField("开始时间", blank=True, null=True)
    end_time = models.DateField("结束时间", blank=True, null=True)
    created_time = models.DateTimeField("Created Time",
                                        auto_now_add=True)

    class Meta:
        app_label = APP_LABEL

    # TODO: @property

    def __str__(self):
        return '{0}-{1}-{2}'.format(self.id, self.title, self.description)



import os

from elasticsearch import Elasticsearch

from . import logger
from .exceptions import ESUtilESClusterException


def init_es():
    ''' 连接ES集群并返回

    如果 集群不可达或不可用, 则抛出异常
    '''
    try:

        hosts = os.environ.get('ES_HOST')

        assert hosts is not None, "未设置ES_HOST"

        port = os.environ.get('ES_PORT')

        assert port is not None, "未设置ES_PORT"

        login = os.environ.get('ES_USER')

        assert login is not None, "未设置ES_USER"

        passwd = os.environ.get('ES_PASSWD')

        assert passwd is not None, "未设置ES_PASSWD"

        es = Elasticsearch(
            hosts=[hosts],
            port=port,
            http_auth=(login, passwd)
        )

        health_status = es.cluster.health().get('status')
        assert health_status != 'red', f"health_status={health_status}"
    except BaseException as err:
        logger.error(err, exc_info=True)
        raise ESUtilESClusterException(err.__str__())
    return es



'''
基于 elasticsearch 定制开发特有功能

通过配置 doc_type、index、mapping、query_mapping 实现定制化查询

示例:
    在 es.constants、es.mapping_constants、es.query_mapping_constants 配置文档类型

    from es import ESDoor

    doc_type = order

    door = ESDoor(doc_type, phone='18575546060', id=1, number='2222222')
    res = door.main()

'''
from .constants import DocType, IndexMapping
from .mapping_constants import MAPPINGS
from .query_mapping_constants import QUERY_MAPPING
from .query_mapping_constants import QUERY_MAPPING_SPECIAL_FUNC

from .es_util.base import register_doc_type

doc_types = {p: v for p, v in DocType.__dict__.items() if not p.startswith('_')}

for doc_type in doc_types.values():
    assert doc_type in IndexMapping, f"文档类型{doc_type}未配置索引"
    assert doc_type in MAPPINGS, f"文档类型{doc_type}未配置 mapping"
    assert doc_type in QUERY_MAPPING, f"文档类型{doc_type}未配置 query_mapping"

    register_doc_type(
        doc_type=doc_type,
        mapping=MAPPINGS.get(doc_type),
        query_mapping=QUERY_MAPPING.get(doc_type),
        special_query_func=QUERY_MAPPING_SPECIAL_FUNC.get(doc_type)
    )


from .es_util.es_door import ESDoor



from .utils import get_seats_by_dis_ids

def decorator_shelfseat_info_in_distribute_list(serializer_data):
    '''
    为发货单列表补充货位信息

    :param serializer_data: [{'id': '发货单id', ...}, ...]
    '''
    dis_ids = [distribute_dict['id'] for distribute_dict in serializer_data]
    seats_info_dict = get_seats_by_dis_ids(dis_ids)
    for distribute_dict in serializer_data:
        storage_record_dict = seats_info_dict.get(distribute_dict['id'], None)
        distribute_dict.update({'storage_info': storage_record_dict})




import unittest
from unittest import mock

from ...es_util.es_door import ESDoor
from ...es_util.es_builder import ESBuilder
from ...es_util.base import __EXAMPLE_DOC_TYPE as EXAMPLE_DOC_TYPE


ESBuilder.__es = mock.Mock()


class TestESDoor(unittest.TestCase):

    def test_builder_in_es_door(self):
        door = ESDoor(
            query_type=EXAMPLE_DOC_TYPE,
            all_number='18512341234',
            main_id=10,
            created_time__before='2022-02-02 00:00:00',
            created_time__after='2022-01-01 00:00:00',
            page=3,
            page_size=10
        )

        builder = ESBuilder(EXAMPLE_DOC_TYPE)
        for query_group in door.es_params:
            builder.addQueryGroup(query_group)
        builder.addQueryGroup(door.es_query_util.get_page_param_group(door.page, door.page_size))

        expected_main_dsl = {
            'query': {
                'bool': {
                    'must': [
                        {
                            'term': {
                                'id': 10
                            }
                        },
                        {
                            'range': {
                                'created_time': {
                                    'lte': '2022-02-02 00:00:00',
                                    'time_zone': '+08:00'
                                }
                            }
                        },
                        {
                            'range': {
                                'created_time': {
                                    'gte': '2022-01-01 00:00:00',
                                    'time_zone': '+08:00'
                                }
                            }
                        },
                        {
                            'bool': {
                                'should': [
                                    {
                                        'term': {
                                            'number': '18512341234'
                                        }
                                    },
                                    {
                                        'nested': {
                                            'path': 'inner',
                                            'query': {
                                                'bool': {
                                                    'filter': {
                                                        'term': {'inner.number': '18512341234'}
                                                    }
                                                }
                                            }
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                }
            }
        }

        self.assertDictEqual(builder.main_dsl, expected_main_dsl)

        expected_paged_dsl = {
            'from': 20,
            'size': 10
        }
        expected_paged_dsl.update(expected_main_dsl)

        self.assertDictEqual(builder.page_dsl, expected_paged_dsl)


if __name__ == '__main__':
    unittest.main()



import logging

from ..core.constants import MemberLevel
from enum import Enum, unique

logger = logging.getLogger(__name__)


DISTRIBUTE_ERROR_CODE_PREFIX = 246000
BASE_DISTRIBUTE_API_CODE = 20103000


class DistributeStatusOwner(object):
    GUIDE = 'guide'
    CUSSERVICE = 'cusservice'
    CUSTOMER = 'customer'
    OUTSTORAGE = 'outstorage'
    # 汉光贩售机
    HGVENDING = 'hgvending'
    # 海淘
    HAITAO = 'haitao'


class DistributeDirection(object):
    GUIDE = 'guide'
    CUSSERVICE = 'cusservice'
    OUTSTORAGE = 'outstorage'
    # 汉光贩售机
    HGVENDING = 'hgvending'
    # 海淘
    HAITAO = 'haitao'


class DistributeStatus(object):
    # 初始态
    INIT = "init"
    # 已打印
    PRINTED = "printed"
    # 客服已提
    CUSSERVICE_PICKED_UP = "cusservice_picked_up"
    # 导购已提
    GUIDE_PICKED_UP = "guide_picked_up"
    # 十层前台
    SERVICE_DESK = "service_desk"
    # 十层仓库
    SERVICE_STORE = "service_store"
    # 已打包发货
    PACKAGED = "packaged"
    # 客服已退
    CUSSERVICE_HOLD = "cusservice_hold"
    # #客户已确收
    CUS_CONFIRM = "cus_confirm"

    # 导购端-客户已确收
    CUS_CONFIRM_AT_GUIDE = "cus_confirm_at_guide"

    # 客服端-客户已确收
    CUS_CONFIRM_AT_CUSSERVICE = "cus_confirm_at_cusservice"

    # 拆单
    SPLIT = "split"
    # 未提已退
    NO_PICKUP_RETURN = "no_pickup_return"
    # 外仓待发货
    OUTSTORAGE_WAIT_PACKAGE = "outstorage_wait_package"
    # 外仓已发货
    OUTSTORAGE_PACKAGED = "outstorage_packaged"
    # 外仓已退
    OUTSTORAGE_RETURN = "outstorage_return"

    # 贩售机初始态
    HGVENDING_INIT = "hgvending_init"
    # 贩售机待提
    HGVENDING_WAIT_PICKUP = "hgvending_wait_pickup"
    # 贩售机客户已自提
    CUS_CONFIRM_AT_HGVENDING = "cus_confirm_at_hgvending"
    # 贩卖机已退
    HGVENDING_NO_PICKUP_RETURN = "hgvending_no_pickup_return"

    # 定金已支付, 由分阶段支付的订单产生
    DEPOSITPAY = "depositpay"
    # 尾款超时
    TAILPAY_OVERTIME = "tailpay_overtime"
    # 定金已退
    DEPOSITPAY_RETURN = "depositpay_return"
    # 海淘初始态
    HAITAO_INIT = "haitao_init"
    # 海淘已推百世快递
    HAITAO_BAISHI = "haitao_baishi"
    # 海淘已退
    HAITAO_RETURN = "haitao_return"
    # 海淘已发货
    HAITAO_PACKAGED = "haitao_packaged"

    # 滞留仓
    SERVICE_STOP = "service_stop"

    # 2022.08 虚拟商品
    # 虚拟商品INIT
    VIRTUAL_INIT = "virtual_init"
    # 虚拟已确收
    CUS_CONFIRM_AT_VIRTUAL = "cus_confirm_at_virtual"

    # 2022.09 特需售后单自提转快递支付运费订单的发货单
    MAKEUP_POSTAGE_PAID = "paid"
    
    # 2023.4 退款单与发货单联动
    # 退款单 售后仓已收 <> 售后仓已收
    # event 售后仓入库
    AFTERSALE_WH = "AftersaleWH"

    NO_SPLIT_STATUS_LIST = [
        INIT,
        HGVENDING_INIT,
        VIRTUAL_INIT,
        PRINTED,
        CUSSERVICE_PICKED_UP,
        GUIDE_PICKED_UP,
        HGVENDING_WAIT_PICKUP,
        CUS_CONFIRM,
        CUS_CONFIRM_AT_GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE,
        CUS_CONFIRM_AT_VIRTUAL,
        CUS_CONFIRM_AT_HGVENDING,
        PACKAGED,
        SERVICE_DESK,
        SERVICE_STORE,
        CUSSERVICE_HOLD,
        NO_PICKUP_RETURN,
        HGVENDING_NO_PICKUP_RETURN,
        OUTSTORAGE_WAIT_PACKAGE,
        OUTSTORAGE_PACKAGED,
        OUTSTORAGE_RETURN,
        DEPOSITPAY,
        TAILPAY_OVERTIME,
        DEPOSITPAY_RETURN,
        HAITAO_INIT,
        HAITAO_BAISHI,
        HAITAO_PACKAGED,
        HAITAO_RETURN,
        SERVICE_STOP,
        MAKEUP_POSTAGE_PAID,
    ]
    

    # 可以被售后集合
    CAN_AFTERSALE_STATUS_LIST = [
        INIT,
        PRINTED,
        CUSSERVICE_PICKED_UP,
        GUIDE_PICKED_UP,
        CUS_CONFIRM,
        CUS_CONFIRM_AT_GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE,
        CUS_CONFIRM_AT_VIRTUAL,
        PACKAGED,
        SERVICE_DESK,
        SERVICE_STORE,
        OUTSTORAGE_WAIT_PACKAGE,
        OUTSTORAGE_PACKAGED,
        SERVICE_STOP,
        MAKEUP_POSTAGE_PAID,
    ]

    ## 初始态集合
    INIT_STATUS_LIST = [
        INIT,
        HGVENDING_INIT,
        VIRTUAL_INIT,
        PRINTED,
        OUTSTORAGE_WAIT_PACKAGE,
    ]
    ## 可以整单退的发货单状态集合
    ENTIRE_AFTERSALE_STATUS_LIST = [
        INIT,
        HGVENDING_INIT,
        PRINTED,
        GUIDE_PICKED_UP,
        SPLIT,
        HGVENDING_WAIT_PICKUP,
        # 补缴运费的商品可以整单退
        MAKEUP_POSTAGE_PAID,
    ]

    BEFORE_cuservice_pick_up = [
        INIT,
        PRINTED,
        NO_PICKUP_RETURN,
        SPLIT,
        CUSSERVICE_HOLD,
        TAILPAY_OVERTIME,
        DEPOSITPAY_RETURN,
    ]

    CAN_SPLIT_STATUS_LIST = [
        INIT,
        PRINTED,
        CUSSERVICE_PICKED_UP,
        GUIDE_PICKED_UP,
        SERVICE_DESK,
        SERVICE_STORE,
        SERVICE_STOP,
    ]
    # For C端显示
    DISTRIBUTE_SHOW_FOR_C = {
        INIT: '备货中',
        HGVENDING_INIT: '备货中',
        PRINTED: "备货中",
        CUSSERVICE_PICKED_UP: "已出库",
        GUIDE_PICKED_UP: "可专柜自提",
        SERVICE_DESK: "可自提",
        SERVICE_STORE: "待发货",
        PACKAGED: "已邮寄",
        OUTSTORAGE_WAIT_PACKAGE: "待发货",
        OUTSTORAGE_PACKAGED: "已邮寄",
        OUTSTORAGE_RETURN: "售后",
        CUSSERVICE_HOLD: "售后",
        CUS_CONFIRM: "已提货",
        CUS_CONFIRM_AT_GUIDE: "已提货",
        CUS_CONFIRM_AT_CUSSERVICE: "已提货",
        CUS_CONFIRM_AT_VIRTUAL: "已完成",  # 2022.8.23 琪琪说改成已完成，目前前端根据这个字段，展示下面的文字描述
        SPLIT: "已拆单",
        NO_PICKUP_RETURN: "售后",
        HGVENDING_WAIT_PICKUP: "待提",
        CUS_CONFIRM_AT_HGVENDING: "已提货",
        HGVENDING_NO_PICKUP_RETURN: "已退",
        DEPOSITPAY: "待支付尾款",
        DEPOSITPAY_RETURN: "定金已退",
        TAILPAY_OVERTIME: "尾款超时",
        HAITAO_INIT: "备货中",
        HAITAO_BAISHI: "备货中",
        HAITAO_PACKAGED: "已发货",
        HAITAO_RETURN: "售后",
        SERVICE_STOP: "待发货",
        MAKEUP_POSTAGE_PAID: "运费已支付",
        AFTERSALE_WH: "售后仓已收",
    }
    # For B端 查询
    DISTRIBUTE_STATUS_CHOICES_FOR_B_QUERY = {
        INIT: '未打印',
        HGVENDING_INIT: '贩售机初始态',
        PRINTED: "已打印",
        CUSSERVICE_PICKED_UP: "客服已提",
        GUIDE_PICKED_UP: "可专柜自提",
        SERVICE_DESK: "自提点已收",
        SERVICE_STORE: "仓库已收",
        PACKAGED: "已邮寄",
        OUTSTORAGE_WAIT_PACKAGE: "外仓待发货",
        OUTSTORAGE_PACKAGED: "外仓已邮寄",
        CUSSERVICE_HOLD: "客服已退",
        CUS_CONFIRM: "已自提",
        CUS_CONFIRM_AT_GUIDE: "导购端已自提",
        CUS_CONFIRM_AT_CUSSERVICE: "客服端已自提",
        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品已提货",
        # SPLIT: "拆单",
        NO_PICKUP_RETURN: "未提已退",
        OUTSTORAGE_RETURN: "外仓已退",
        HGVENDING_WAIT_PICKUP: "贩售机待提",
        CUS_CONFIRM_AT_HGVENDING: "贩售机端已提货",
        HGVENDING_NO_PICKUP_RETURN: "贩售机已退",
        DEPOSITPAY: "定金已付",
        DEPOSITPAY_RETURN: "定金已退",
        TAILPAY_OVERTIME: "尾款超时",
        HAITAO_INIT: "海淘备货中",
        HAITAO_BAISHI: "海淘已推订单",
        HAITAO_PACKAGED: "海淘已发货",
        HAITAO_RETURN: "海淘已退",
        SERVICE_STOP: "滞留仓已收",
        MAKEUP_POSTAGE_PAID: "运费已支付",
        AFTERSALE_WH: "售后仓已收",
    }

    DISTRIBUTE_MESSAGE_DICT_FOR_C = {
        INIT: "专柜已收到您的订单，正在为您备货。",
        SERVICE_STORE: "商品已从专柜提货，正在为您准备邮寄。",
        SERVICE_DESK: "商品已备好，您可至【汉光百货顾客服务中心（一层北客梯旁）】领取。",
        CUSSERVICE_PICKED_UP: "商品已经出库。",
        GUIDE_PICKED_UP: "商品已备好，请至【汉光百货shop_location_desc shop_name专柜】领取。",
        PACKAGED: "已交付express_name快递，运单号为shipping_number。",
        OUTSTORAGE_WAIT_PACKAGE: "待外仓发货。",
        OUTSTORAGE_PACKAGED: "已交付express_name快递，运单号为shipping_number。",
        CUS_CONFIRM: "商品已提走，期待您下次光临。",
        CUS_CONFIRM_AT_GUIDE: "商品已提走，期待您下次光临。",
        CUS_CONFIRM_AT_CUSSERVICE: "商品已提走，期待您下次光临。",
        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品已提货，期待您下次光临。",
        CUSSERVICE_HOLD: "商品已售后。",
        NO_PICKUP_RETURN: "商品已售后。",
        OUTSTORAGE_RETURN: "外仓已售后。",
        HGVENDING_NO_PICKUP_RETURN: "订单超时30分钟自动退单，如有需要请再扫码下单。",
        DEPOSITPAY: "定金已支付。",
        DEPOSITPAY_RETURN: "您的商品已退还定金",
        TAILPAY_OVERTIME: "未按时支付尾款，您的订单已关闭",
        HAITAO_INIT: "海淘备货中。",
        HAITAO_BAISHI: "海淘备货中。",
        HAITAO_PACKAGED: "海淘已发货。",
        HAITAO_RETURN: "海淘已售后。",
        SERVICE_STOP: "暂时无法发货，商品滞留。",
        MAKEUP_POSTAGE_PAID: "订单自提转快递运费已支付。",
    }

    DISTRIBUTE_STATUS_CHOICES = (
        (INIT, '未打印'),
        (HGVENDING_INIT, '贩售机初始态'),
        (PRINTED, "已打印"),
        (CUSSERVICE_PICKED_UP, "客服已提"),
        (GUIDE_PICKED_UP, "可专柜自提"),
        (SERVICE_DESK, "前台已收"),
        (SERVICE_STORE, "仓库已收"),
        (PACKAGED, "已邮寄"),
        (OUTSTORAGE_WAIT_PACKAGE, "外仓待发货"),
        (OUTSTORAGE_PACKAGED, "外仓已邮寄"),
        (CUSSERVICE_HOLD, "客服已退"),
        (CUS_CONFIRM, "已自提"),
        (CUS_CONFIRM_AT_GUIDE, "导购端已自提"),
        (CUS_CONFIRM_AT_CUSSERVICE, "客服端已自提"),
        (SPLIT, "已拆单"),
        (NO_PICKUP_RETURN, "未提已退"),
        (OUTSTORAGE_RETURN, "外仓已退"),
        (HGVENDING_WAIT_PICKUP, "贩售机待提"),
        (CUS_CONFIRM_AT_HGVENDING, "贩售机端已提货"),
        (HGVENDING_NO_PICKUP_RETURN, "贩售机端已退"),
        (DEPOSITPAY, "定金已付"),
        (DEPOSITPAY_RETURN, "定金已退"),
        (TAILPAY_OVERTIME, "尾款超时"),
        (HAITAO_INIT, "海淘备货中"),
        (HAITAO_BAISHI, "海淘已推订单"),
        (HAITAO_PACKAGED, "海淘已发货"),
        (HAITAO_RETURN, "海淘已退"),
        (SERVICE_STOP, "滞留仓已收"),
        (VIRTUAL_INIT, "虚拟商品初始态"),
        (CUS_CONFIRM_AT_VIRTUAL, "虚拟商品已提货"),
        (MAKEUP_POSTAGE_PAID, "运费已支付"),
        (AFTERSALE_WH, "售后仓已收"),
    )

    DISTRIBUTE_STATUS_OWNER_DICT = {
        INIT: DistributeStatusOwner.GUIDE,
        VIRTUAL_INIT: DistributeStatusOwner.CUSTOMER,
        HGVENDING_INIT: DistributeStatusOwner.HGVENDING,
        PRINTED: DistributeStatusOwner.GUIDE,
        GUIDE_PICKED_UP: DistributeStatusOwner.GUIDE,
        NO_PICKUP_RETURN: DistributeStatusOwner.GUIDE,
        CUSSERVICE_PICKED_UP: DistributeStatusOwner.CUSSERVICE,
        HGVENDING_WAIT_PICKUP: DistributeStatusOwner.HGVENDING,
        SERVICE_DESK: DistributeStatusOwner.CUSSERVICE,
        SERVICE_STORE: DistributeStatusOwner.CUSSERVICE,
        CUSSERVICE_HOLD: DistributeStatusOwner.CUSSERVICE,
        CUS_CONFIRM: DistributeStatusOwner.CUSTOMER,
        CUS_CONFIRM_AT_GUIDE: DistributeStatusOwner.CUSTOMER,
        CUS_CONFIRM_AT_CUSSERVICE: DistributeStatusOwner.CUSTOMER,
        CUS_CONFIRM_AT_VIRTUAL: DistributeStatusOwner.CUSTOMER,
        CUS_CONFIRM_AT_HGVENDING: DistributeStatusOwner.CUSTOMER,
        PACKAGED: DistributeStatusOwner.CUSTOMER,
        OUTSTORAGE_WAIT_PACKAGE: DistributeStatusOwner.OUTSTORAGE,  # 外仓
        OUTSTORAGE_PACKAGED: DistributeStatusOwner.CUSTOMER,
        OUTSTORAGE_RETURN: DistributeStatusOwner.OUTSTORAGE,  # 外仓
        HGVENDING_NO_PICKUP_RETURN: DistributeStatusOwner.HGVENDING,
        DEPOSITPAY: DistributeStatusOwner.GUIDE,
        DEPOSITPAY_RETURN: DistributeStatusOwner.GUIDE,
        TAILPAY_OVERTIME: DistributeStatusOwner.GUIDE,
        HAITAO_INIT: DistributeStatusOwner.HAITAO,
        HAITAO_BAISHI: DistributeStatusOwner.HAITAO,
        HAITAO_PACKAGED: DistributeStatusOwner.HAITAO,
        HAITAO_RETURN: DistributeStatusOwner.HAITAO,
        SERVICE_STOP: DistributeStatusOwner.CUSSERVICE,
        MAKEUP_POSTAGE_PAID: DistributeStatusOwner.CUSTOMER,
    }

    DISTRIBUTE_STATUS_DIRECTION_DICT = {
        INIT: DistributeDirection.GUIDE,
        HGVENDING_INIT: DistributeDirection.HGVENDING,
        PRINTED: DistributeDirection.GUIDE,
        GUIDE_PICKED_UP: DistributeDirection.GUIDE,
        NO_PICKUP_RETURN: DistributeDirection.GUIDE,
        CUSSERVICE_PICKED_UP: DistributeDirection.CUSSERVICE,
        SERVICE_DESK: DistributeDirection.CUSSERVICE,
        SERVICE_STORE: DistributeDirection.CUSSERVICE,
        CUSSERVICE_HOLD: DistributeDirection.CUSSERVICE,
        CUS_CONFIRM: DistributeDirection.CUSSERVICE,
        CUS_CONFIRM_AT_GUIDE: DistributeDirection.GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE: DistributeDirection.CUSSERVICE,
        CUS_CONFIRM_AT_HGVENDING: DistributeDirection.HGVENDING,
        PACKAGED: DistributeDirection.CUSSERVICE,
        OUTSTORAGE_WAIT_PACKAGE: DistributeDirection.OUTSTORAGE,  # 外仓
        OUTSTORAGE_PACKAGED: DistributeDirection.OUTSTORAGE,
        OUTSTORAGE_RETURN: DistributeDirection.OUTSTORAGE,  # 外仓
        HGVENDING_WAIT_PICKUP: DistributeDirection.HGVENDING,  # 汉光贩售机
        HGVENDING_NO_PICKUP_RETURN: DistributeDirection.HGVENDING,
        DEPOSITPAY: DistributeDirection.GUIDE,
        DEPOSITPAY_RETURN: DistributeDirection.GUIDE,
        TAILPAY_OVERTIME: DistributeDirection.GUIDE,

        HAITAO_INIT: DistributeDirection.HAITAO,
        HAITAO_BAISHI: DistributeDirection.HAITAO,
        HAITAO_PACKAGED: DistributeDirection.HAITAO,
        HAITAO_RETURN: DistributeDirection.HAITAO,
        SERVICE_STOP: DistributeDirection.CUSSERVICE,

        MAKEUP_POSTAGE_PAID: DistributeDirection.CUSSERVICE,
    }

    # 终结状态列表
    TERMINATE_STATUS_LIST = [
        PACKAGED,
        CUSSERVICE_HOLD,
        CUS_CONFIRM,
        CUS_CONFIRM_AT_GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE,
        CUS_CONFIRM_AT_VIRTUAL,
        CUS_CONFIRM_AT_HGVENDING,
        SPLIT,
        NO_PICKUP_RETURN,
        HGVENDING_NO_PICKUP_RETURN,
        # 外仓
        OUTSTORAGE_PACKAGED,
        OUTSTORAGE_RETURN,
        # 分阶段支付
        DEPOSITPAY_RETURN,
        TAILPAY_OVERTIME,
        # 海淘
        HAITAO_PACKAGED,
        HAITAO_RETURN,
        MAKEUP_POSTAGE_PAID,
    ]

    # 非最终状态
    UN_TERMINATE_STATUS_LIST = [
        SERVICE_DESK,
        SERVICE_STORE,
        PRINTED,
        INIT,
        GUIDE_PICKED_UP,
        CUSSERVICE_PICKED_UP,
        SERVICE_STOP,
    ]

    # 由售后产生的终结发货单状态列表(含SPLIT)
    AFTERSALE_TERMINATE_STATUS_LIST = [
        CUSSERVICE_HOLD,
        SPLIT,
        NO_PICKUP_RETURN,
        HGVENDING_NO_PICKUP_RETURN,
        OUTSTORAGE_RETURN,
        DEPOSITPAY_RETURN,
        HAITAO_RETURN,
    ]

    # 可换货发货单状态
    EXCHANGE_TERMINATE_STATUS_LIST = [
        PACKAGED,
        CUS_CONFIRM,
        CUS_CONFIRM_AT_GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE,
        OUTSTORAGE_PACKAGED,
    ]

    # 未售后的不支持修改配送方式的发货单状态
    CAN_NOT_MODY_SHIPPING_METHOD_DISTRIBUTE_STATUS_LIST = [
        CUS_CONFIRM_AT_GUIDE,
        CUS_CONFIRM_AT_CUSSERVICE,
        PACKAGED,
        OUTSTORAGE_PACKAGED,
        OUTSTORAGE_WAIT_PACKAGE,
        SPLIT,
    ]

    # 未出汉光的发货单状态
    DISTRIBUTE_STATUS_IN_HG_LIST = [
        INIT,
        PRINTED,
        CUSSERVICE_PICKED_UP,
        SERVICE_DESK,
        SERVICE_STORE,
        SPLIT,
        OUTSTORAGE_WAIT_PACKAGE,
        HGVENDING_INIT,
        HGVENDING_WAIT_PICKUP,
        TAILPAY_OVERTIME,
        HAITAO_INIT,
        HAITAO_BAISHI,
        SERVICE_STOP,
    ]

    # 未出专柜
    DISTRIBUTE_STATUS_IN_SHOP_LIST = [
        INIT,
        PRINTED,
        OUTSTORAGE_WAIT_PACKAGE,
    ]


class __ShippingMethod(object):
    # 普通快递
    EXPRESS = 'express'
    # 自提
    SELF_SERVICE = 'self_service'
    # 闪送
    FLASH_DELIVERY = 'flash_delivery'

    CHOICES = (
        (EXPRESS, "快递发货"),
        (SELF_SERVICE, "到店自提"),
        (FLASH_DELIVERY, "闪送"),
    )

    def __getattribute__(self, name: str):
        value = super().__getattribute__(name)
        logger.warning("sparrow_distribute.constants.ShippingMethod 已废弃, 请使用 sparrow_order_lib.core.constants.ShippingMethod 替换")
        return value


ShippingMethod = __ShippingMethod()


class DistributeEventConstKey(object):
    FROM_STATUS_LIST_KEY = "from_status_list"
    TO_STATUS_KEY = "to_status"
    TO_STATE_MAP_KEY = "to_state_map"


class MessageLevel(object):
    FOR_CUSTOMER = 1
    FOR_CUSSERVICE = 2
    FOR_DEV = 3


class DistributeSyncType(object):
    MEMBER = "MEMBER"
    VISITOR = "VISITOR"


# 发货单操作事件名称常量
class DistributeEventConst(object):
    NEW_ADD = "new_add"
    PRINT = "print"
    GUIDE_PRINT = "guide_print"
    PRINT_AND_OVERWRITE_EXPRESS = "print_and_overwrite_express"
    CANCEL_PRINT = "cancel_print"
    CUSSERVICE_PICK_UP = "cusservice_pick_up"

    CUSSERVICE_PICK_UP_BY_DEVICEC_ONLY_FOR_LOG = "cusservice_pick_up_by_device_only_for_log"

    GUIDE_PICK_UP = "guide_pick_up"
    SERVICESTORE_PICK_UP = "servicestore_pick_up"
    SERVICEDESK_PICK_UP = "servicedesk_pick_up"

    PACKAGE = "package"
    PACKAGE_BY_DEVICEC_ONLY_FOR_LOG = "package_by_device_only_for_log"
    CANCEL_PACKAGE = "cancel_package"
    SERVICESTOP_CANCEL_PACKAGE = "service_stop_cancel_package"

    CUS_CONFIRM = "cus_confirm"
    CUS_CONFIRM_AT_GUIDE = "cus_confirm_at_guide"
    CUS_CONFIRM_AT_CUSSERVICE = "cus_confirm_at_cusservice"

    TO_WAIT_OUTSTORAGE_PACKAGE = "to_wait_outstorage_package"
    CANCEL_WAIT_OUTSTORAGE_PACKAGE = "cancel_wait_outstorage_package"
    OUTSTORAGE_PACKAGE = "outstorage_package"
    CANCEL_OUTSTORAGE_PACKAGE = "cancel_outstorage_package"

    OPEN_AFTERSALE = "open_aftersale"
    CLOSE_AFTERSALE = "close_aftersale"
    COMPLETE_AFTERSALE = "complete_aftersale"
    CREATE_EXPRESS = "create_express"
    CANCEL_EXPRESS = "cancel_express"
    MODIFY_SHIPPING_NUMBER = "modfiy_shipping_number"
    MERGE_EXPRESS = "merge_express"
    MERGE_EXPRESS_MAIN = "merge_express_main"
    # 客服退还货物给专柜事件
    CUSSERVICE_RETURN = "cusservice_return"
    OUTSTORAGE_RETURN = "outstorage_return"
    ADD_SHIPPING_ADDRESS = "add_shipping_address"
    UPDATE_SHIPPING_METHOD = "update_shipping_method"
    UPDATE_SHIPPING_METHOD_FOR_MERGE = "update_shipping_method_for_merge"
    ADD_GUIDE_NOTE = "add_guide_note"
    BLOCK_SUCC = "block_succ"
    BLOCK_FAILED = "block_failed"
    MODIFY_SHIPPING_NUMBER_FAILED = "modify_shipping_number_failed"

    # 货架管理 出库/入库/更改货位/合并货位
    STORAGE_IN = "storage_in"
    STORAGE_OUT = "storage_out"
    STORAGE_SEAT_CHANGE = "storage_seat_change"
    STORAGE_SEAT_MERGE = "storage_seat_merge"

    # 换货
    EXCHANGE_OPEN = "exchange_open"
    EXCHANGE_CANCEL = "exchange_cancel"
    EXCHANGE_DONE = "exchange_done"
    # 汉光贩卖机提货
    HGVENDING_PICKED_UP = "hgvending_pick_up"
    # 汉光贩卖机-客户自提
    CUS_CONFIRM_AT_HGVENDING = "cus_confirm_at_hgvending"

    # 海淘打包
    HAITAO_PACKAGE = "haitao_package"

    # 海淘推订单（现在这个事件是海淘通用的推订单，不要被baishi这个名字迷惑）
    HAITAO_PUSHDATA_2_BAISHI = "haitao_pushdata_2_baishi"
    # 海淘取消订单（现在这个事件是海淘通用的取消订单，不要被baishi这个名字迷惑）
    HAITAO_CANCEL_FROM_BAISHI = "haitao_cancel_from_baishi"
    # 报错日志
    ERROR_MESSAGE = "error_message"
    # 普通日志
    NORMAL_MESSAGE = "normal_message"

    # 付尾款
    PAY_TAIL = "pay_tail"
    # 退定金
    RETURN_DEPOSIT = "return_deposit"
    # 尾款超时
    OVERTIME_TAILPAY = "overtime_tailpay"
    # 更新发货时间
    UPDATE_DELIVER_TIME = "update_deliver_time"
    # 合单催配送提醒
    PUSH_GUIDE_ASSIGN = "push_guide_assign"

    # 2022.08 特殊售后单需求
    AFSPLUS_OPEN = "afsplus_open"
    AFSPLUS_CLOSE = "afsplus_close"
    AFSPLUS_COMPLETE = "afsplus_complete"
    AFSPLUS_S2E = "afsplus_s2e"  # 自提转快递
    AFSPLUS_E2S = "afsplus_e2s"  # 快递转自提
    AFSPLUS_TO_EXPRESS = "afsplus_2e"  # 转快递
    AFSPLUS_TO_SELF_SERVICE = "afsplus_2s"

    # 2022.08 虚拟商品确收
    CUS_CONFIRM_AT_VIRTUAL = "cus_confirm_at_virtual"
    
    CLOUDPRINT_SHIPPING_NUMBER = "cloudprint_shipping_number"
    
    # 2023.4 售后仓入库
    ENTER_AFTERSALE_WH = 'enter_aftersale_wh'
    # 2023.4 手动拆单
    MANUAL_SPLIT = "manual_split"
    # 2023.4 超卖
    OVERSELL = "oversell"
    # 2023.10 延迟发货
    DELAY_DELIVER = "delay_deliver"

    LOG = "log"

    FROM_TO_MAP = {
        PRINT: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.PACKAGED,
                DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.CUS_CONFIRM,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATE_MAP_KEY: {
                DistributeStatus.INIT: DistributeStatus.PRINTED,
            },
        },
        GUIDE_PRINT: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.GUIDE_PICKED_UP
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SPLIT
        },
        PRINT_AND_OVERWRITE_EXPRESS: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.CUS_CONFIRM,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.PACKAGED,
            ],
            DistributeEventConstKey.TO_STATE_MAP_KEY: {
                DistributeStatus.INIT: DistributeStatus.PRINTED,
            },
        },
        CANCEL_PRINT: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.PRINTED],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.INIT,
        },
        CUSSERVICE_PICK_UP: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.AFTERSALE_WH,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUSSERVICE_PICKED_UP,
        },
        GUIDE_PICK_UP: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.INIT, DistributeStatus.PRINTED],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.GUIDE_PICKED_UP,
        },
        HGVENDING_PICKED_UP: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.HGVENDING_INIT],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HGVENDING_WAIT_PICKUP,
        },
        SERVICESTORE_PICK_UP: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT, 
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP, 
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.AFTERSALE_WH,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SERVICE_STORE,
        },
        SERVICEDESK_PICK_UP: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.AFTERSALE_WH,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SERVICE_DESK,
        },
        PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.PACKAGED,
        },
        CANCEL_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.PACKAGED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        SERVICESTOP_CANCEL_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.PACKAGED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_STORE],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SERVICE_STOP,
        },
        CUS_CONFIRM_AT_GUIDE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP, DistributeStatus.SERVICE_DESK],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM_AT_GUIDE,
        },
        CUS_CONFIRM_AT_CUSSERVICE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP, DistributeStatus.SERVICE_DESK,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
        },
        CUS_CONFIRM_AT_HGVENDING: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.HGVENDING_WAIT_PICKUP],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM_AT_HGVENDING,
        },

        TO_WAIT_OUTSTORAGE_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT, DistributeStatus.PRINTED],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
        },
        CANCEL_WAIT_OUTSTORAGE_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.OUTSTORAGE_WAIT_PACKAGE],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        OUTSTORAGE_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                # DistributeStatus.INIT,
                # DistributeStatus.PRINTED,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.OUTSTORAGE_PACKAGED,
        },
        CANCEL_OUTSTORAGE_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                # DistributeStatus.OUTSTORAGE_PACKAGED
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },

        OPEN_AFTERSALE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP, DistributeStatus.SERVICE_STORE, DistributeStatus.SERVICE_DESK,
                DistributeStatus.PACKAGED,
                DistributeStatus.CUS_CONFIRM,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.PACKAGED, DistributeStatus.SERVICE_STOP,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.HGVENDING_INIT, DistributeStatus.HGVENDING_WAIT_PICKUP, DistributeStatus.CUS_CONFIRM_AT_HGVENDING,
                DistributeStatus.HAITAO_INIT,
                DistributeStatus.DEPOSITPAY,
                DistributeStatus.TAILPAY_OVERTIME,
                DistributeStatus.MAKEUP_POSTAGE_PAID,
                DistributeStatus.AFTERSALE_WH,
            ],

            DistributeEventConstKey.TO_STATE_MAP_KEY: {
                DistributeStatus.INIT: DistributeStatus.SPLIT,
                DistributeStatus.PRINTED: DistributeStatus.SPLIT,
                DistributeStatus.CUSSERVICE_PICKED_UP: DistributeStatus.SPLIT,
                DistributeStatus.GUIDE_PICKED_UP: DistributeStatus.SPLIT,
                DistributeStatus.SERVICE_STORE: DistributeStatus.SPLIT,
                DistributeStatus.SERVICE_DESK: DistributeStatus.SPLIT,
                DistributeStatus.PACKAGED: None,
                DistributeStatus.CUS_CONFIRM: None,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE: None,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE: None,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL: None,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE: DistributeStatus.SPLIT,
                DistributeStatus.OUTSTORAGE_PACKAGED: None,
                DistributeStatus.HGVENDING_INIT: None,
                DistributeStatus.HGVENDING_WAIT_PICKUP: None,
                DistributeStatus.CUS_CONFIRM_AT_HGVENDING: None,
                DistributeStatus.HAITAO_INIT: None,
                DistributeStatus.DEPOSITPAY: None,
                DistributeStatus.SERVICE_STOP: DistributeStatus.SPLIT,
                # DistributeStatus.HAITAO_INIT: None,
                DistributeStatus.TAILPAY_OVERTIME: None,
                DistributeStatus.MAKEUP_POSTAGE_PAID: None,
                DistributeStatus.AFTERSALE_WH: None,
            },
        },
        CLOSE_AFTERSALE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.CUS_CONFIRM,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.PACKAGED,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.OUTSTORAGE_RETURN,
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.HGVENDING_INIT,
                DistributeStatus.HGVENDING_WAIT_PICKUP,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.DEPOSITPAY,
                DistributeStatus.TAILPAY_OVERTIME,
                DistributeStatus.MAKEUP_POSTAGE_PAID,
                DistributeStatus.AFTERSALE_WH,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        COMPLETE_AFTERSALE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.OUTSTORAGE_RETURN,
                DistributeStatus.HGVENDING_INIT,
                DistributeStatus.HGVENDING_WAIT_PICKUP,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                DistributeStatus.HAITAO_INIT,
                DistributeStatus.DEPOSITPAY,
                DistributeStatus.TAILPAY_OVERTIME,
                DistributeStatus.MAKEUP_POSTAGE_PAID,
                DistributeStatus.PACKAGED,
                DistributeStatus.AFTERSALE_WH,
                DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
            ],
            DistributeEventConstKey.TO_STATE_MAP_KEY: {
                DistributeStatus.INIT: DistributeStatus.NO_PICKUP_RETURN,
                DistributeStatus.PRINTED: DistributeStatus.NO_PICKUP_RETURN,
                DistributeStatus.GUIDE_PICKED_UP: DistributeStatus.NO_PICKUP_RETURN,
                DistributeStatus.HGVENDING_INIT: DistributeStatus.HGVENDING_NO_PICKUP_RETURN,
                DistributeStatus.HGVENDING_WAIT_PICKUP: DistributeStatus.HGVENDING_NO_PICKUP_RETURN,
                DistributeStatus.HAITAO_INIT: DistributeStatus.HAITAO_RETURN,
                # DistributeStatus.OUTSTORAGE_WAIT_PACKAGE: DistributeStatus.OUTSTORAGE_RETURN
                # DistributeStatus.CUSSERVICE_PICKED_UP:DistributeStatus.CUSSERVICE_HOLD,
                # DistributeStatus.SERVICE_DESK:DistributeStatus.CUSSERVICE_HOLD,
                # DistributeStatus.SERVICE_STORE:DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.DEPOSITPAY: DistributeStatus.DEPOSITPAY_RETURN,
                DistributeStatus.TAILPAY_OVERTIME: DistributeStatus.DEPOSITPAY_RETURN,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL: None,
                DistributeStatus.MAKEUP_POSTAGE_PAID: None,
                DistributeStatus.PACKAGED: None,
                DistributeStatus.OUTSTORAGE_PACKAGED: None,
                DistributeStatus.OUTSTORAGE_RETURN: None,
                DistributeStatus.CUSSERVICE_HOLD: None,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE: None,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE: None,
            }
        },
        LOG: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.OUTSTORAGE_RETURN,
                DistributeStatus.HGVENDING_INIT,
                DistributeStatus.HGVENDING_WAIT_PICKUP,
                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                DistributeStatus.HAITAO_INIT,
                DistributeStatus.HAITAO_BAISHI,
                DistributeStatus.HAITAO_PACKAGED,
                DistributeStatus.HAITAO_RETURN,
                DistributeStatus.DEPOSITPAY,
                DistributeStatus.TAILPAY_OVERTIME,
                DistributeStatus.MAKEUP_POSTAGE_PAID,
                DistributeStatus.PACKAGED,
                DistributeStatus.AFTERSALE_WH,
                DistributeStatus.CUSSERVICE_HOLD,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        CREATE_EXPRESS: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.PACKAGED,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        CANCEL_EXPRESS: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.PACKAGED,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        MODIFY_SHIPPING_NUMBER: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.PACKAGED,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        CUSSERVICE_RETURN: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.AFTERSALE_WH,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUSSERVICE_HOLD,  # 客服已退
        },
        OUTSTORAGE_RETURN: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.OUTSTORAGE_RETURN,  # 外仓已退
        },
        ADD_GUIDE_NOTE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.PACKAGED,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
                DistributeStatus.CUS_CONFIRM,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        EXCHANGE_OPEN: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [  # 仅支持发货单邮寄或者自提后进行换货申请，外仓不支持换货
                DistributeStatus.PACKAGED,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,  # 发货单状态不变更
        },
        EXCHANGE_CANCEL: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [  # 仅支持发货单邮寄或者自提后进行换货申请，外仓不支持换货
                DistributeStatus.PACKAGED,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,  # 发货单状态不变更
        },
        EXCHANGE_DONE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [  # 仅支持发货单邮寄或者自提后进行换货申请，外仓不支持换货
                DistributeStatus.PACKAGED,
                DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                DistributeStatus.OUTSTORAGE_PACKAGED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,  # 发货单状态不变更
        },
        RETURN_DEPOSIT: {  # 分阶段支付--退定金
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.DEPOSITPAY,
                DistributeStatus.TAILPAY_OVERTIME,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.DEPOSITPAY_RETURN
        },
        OVERTIME_TAILPAY: {  # 分阶段支付--尾款超时
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.DEPOSITPAY,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.TAILPAY_OVERTIME,
        },
        PAY_TAIL: {  # 分阶段支付--支付尾款
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.DEPOSITPAY,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.INIT,
        },
        HAITAO_PACKAGE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.HAITAO_BAISHI,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_PACKAGED,
        },
        HAITAO_PUSHDATA_2_BAISHI: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.HAITAO_INIT,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_BAISHI,
        },
        HAITAO_CANCEL_FROM_BAISHI: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.HAITAO_BAISHI,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_INIT,
        },

        PUSH_GUIDE_ASSIGN: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.PRINTED,
                DistributeStatus.INIT,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },

        # 2022.08 特殊售后单需求
        AFSPLUS_OPEN: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_CLOSE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_COMPLETE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_S2E: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_E2S: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_TO_EXPRESS: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        AFSPLUS_TO_SELF_SERVICE: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.GUIDE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STORE,
                DistributeStatus.SERVICE_STOP,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },

        # 2022.08 虚拟商品确收事件
        CUS_CONFIRM_AT_VIRTUAL: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.VIRTUAL_INIT,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
        },
        # 2022.12.16 云打印面单
        CLOUDPRINT_SHIPPING_NUMBER:{
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.PACKAGED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None
        },
        # 2023.4 售后仓入库 (位于客服处的发货单，可以执行售后仓入库)
        ENTER_AFTERSALE_WH: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.SERVICE_STORE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.AFTERSALE_WH,
        },
        MANUAL_SPLIT: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.CUSSERVICE_PICKED_UP,
                DistributeStatus.SERVICE_DESK,
                DistributeStatus.SERVICE_STOP,
                DistributeStatus.SERVICE_STORE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SPLIT,
        },
        OVERSELL: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
        DELAY_DELIVER: {
            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                DistributeStatus.INIT,
                DistributeStatus.PRINTED,
                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
            ],
            DistributeEventConstKey.TO_STATUS_KEY: None,
        },
    }

    EVENT_TO_BUTTONNAME_MAP = {
        NEW_ADD: "新增发货单",
        PRINT: '打印',
        GUIDE_PRINT: '导购打单',
        PRINT_AND_OVERWRITE_EXPRESS: '更新运单并打印',
        CANCEL_PRINT: "清除打印",
        CUSSERVICE_PICK_UP: "客服提货",
        CUSSERVICE_PICK_UP_BY_DEVICEC_ONLY_FOR_LOG: "设备提货",
        GUIDE_PICK_UP: "导购提货",
        SERVICESTORE_PICK_UP: "B3仓库收货",
        SERVICEDESK_PICK_UP: "自提点入库",

        PACKAGE: "快递发货",
        CANCEL_PACKAGE: "取消发货",
        PACKAGE_BY_DEVICEC_ONLY_FOR_LOG: "设备发货",
        SERVICESTOP_CANCEL_PACKAGE: "滞留仓入库",

        TO_WAIT_OUTSTORAGE_PACKAGE: "外仓待发货",
        CANCEL_WAIT_OUTSTORAGE_PACKAGE: "取消外仓待发货",
        OUTSTORAGE_PACKAGE: "外仓发货",
        CANCEL_OUTSTORAGE_PACKAGE: "取消外仓发货",

        CUS_CONFIRM: "客人自提",
        CUS_CONFIRM_AT_GUIDE: "导购端客人自提",
        CUS_CONFIRM_AT_CUSSERVICE: "客服端客人自提",

        OPEN_AFTERSALE: "新建退款单",
        CLOSE_AFTERSALE: "关闭售后",
        COMPLETE_AFTERSALE: "完成售后",
        CREATE_EXPRESS: "快递下单",
        CANCEL_EXPRESS: "取消快递下单",
        MODIFY_SHIPPING_NUMBER: "修改运单号",
        UPDATE_SHIPPING_METHOD_FOR_MERGE: "合单-修改发货方式",
        MERGE_EXPRESS: "合单-修改运单号-辅单",
        MERGE_EXPRESS_MAIN: "合单-修改运单号-主单",
        CUSSERVICE_RETURN: "客服还货",
        OUTSTORAGE_RETURN: "外仓还货",
        ADD_SHIPPING_ADDRESS: "新增发货地址",
        UPDATE_SHIPPING_METHOD: "修改发货方式",
        ADD_GUIDE_NOTE: "导购备注",
        BLOCK_SUCC: "拦截成功",
        BLOCK_FAILED: "拦截失败",
        MODIFY_SHIPPING_NUMBER_FAILED: "修改运单号失败",
        # 货架管理
        STORAGE_IN: "入库",
        STORAGE_OUT: "出库",
        STORAGE_SEAT_CHANGE: "更改货位",
        STORAGE_SEAT_MERGE: "更改货位",

        # 换货
        EXCHANGE_OPEN: "申请换货",
        EXCHANGE_CANCEL: "取消换货",
        EXCHANGE_DONE: "换货完成",
        # 汉光贩卖机提货
        HGVENDING_PICKED_UP: "贩售机提货",
        # 汉光贩卖机-客人自提
        CUS_CONFIRM_AT_HGVENDING: "贩售机端客人自提",
        # 报错日志
        ERROR_MESSAGE: "错误日志",
        # 分阶段支付相关
        PAY_TAIL: "支付尾款",
        RETURN_DEPOSIT: "退定金",
        OVERTIME_TAILPAY: "尾款支付超时",
        # 普通日志
        NORMAL_MESSAGE: "普通日志",
        HAITAO_PACKAGE: "海淘已发货",
        # 海淘推数据给百世
        HAITAO_PUSHDATA_2_BAISHI: "海淘推订单",
        HAITAO_CANCEL_FROM_BAISHI: "海淘取消订单",
        UPDATE_DELIVER_TIME: "更新发货时间",
        # 合单催配货消息功能
        PUSH_GUIDE_ASSIGN: "合单催配送提醒",

        # 特殊售后单
        AFSPLUS_OPEN: "申请特殊售后",
        AFSPLUS_CLOSE: "取消特殊售后",
        AFSPLUS_COMPLETE: "完成特殊售后",
        AFSPLUS_S2E: "自提转快递",  # 自提转快递
        AFSPLUS_E2S: "快递转自提",  # 快递转自提
        AFSPLUS_TO_EXPRESS: "转快递",
        AFSPLUS_TO_SELF_SERVICE: "转自提",

        # 虚拟商品
        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品确收",
        CLOUDPRINT_SHIPPING_NUMBER: "打印面单",
        # 售后
        ENTER_AFTERSALE_WH: "售后仓入库",
        MANUAL_SPLIT: "手动拆单",
        OVERSELL: "超卖",
        DELAY_DELIVER: "延迟发货",
        LOG: "日志",
    }


def get_available_event_list(distribute_status):
    '''
    根据发货单的状态，获取可进行的操作事件列表
    '''
    available_event_list = []
    if not distribute_status:
        return available_event_list
    special_event_list = [
        DistributeEventConst.OPEN_AFTERSALE,
        DistributeEventConst.CLOSE_AFTERSALE,
        DistributeEventConst.COMPLETE_AFTERSALE,
        DistributeEventConst.CREATE_EXPRESS,
        DistributeEventConst.CANCEL_EXPRESS,
        DistributeEventConst.ADD_GUIDE_NOTE,
        DistributeEventConst.CUSSERVICE_RETURN,
        DistributeEventConst.OUTSTORAGE_RETURN,
        DistributeEventConst.EXCHANGE_OPEN,
        DistributeEventConst.EXCHANGE_CANCEL,
        DistributeEventConst.EXCHANGE_DONE,
        DistributeEventConst.HGVENDING_PICKED_UP,
        DistributeEventConst.CUS_CONFIRM_AT_HGVENDING,
        DistributeEventConst.PAY_TAIL,
        DistributeEventConst.RETURN_DEPOSIT,
        DistributeEventConst.OVERTIME_TAILPAY,
        DistributeEventConst.HAITAO_PACKAGE,
        DistributeEventConst.UPDATE_DELIVER_TIME,
        DistributeEventConst.CLOUDPRINT_SHIPPING_NUMBER
    ]
    from_to_map = DistributeEventConst.FROM_TO_MAP
    for event_name, event_dict in from_to_map.items():
        if DistributeEventConstKey.FROM_STATUS_LIST_KEY not in event_dict:
            continue
        from_status_list = event_dict.get(DistributeEventConstKey.FROM_STATUS_LIST_KEY)
        if distribute_status in from_status_list and event_name not in special_event_list:
            event_str = event_name + "|" + DistributeEventConst.EVENT_TO_BUTTONNAME_MAP.get(event_name, event_name)
            available_event_list.append(event_str)
    return available_event_list


class DistributeCommonConst(object):
    NAME_PRIFIX = "包裹"

    MEMBER_CHOICES = (
        (MemberLevel.MEMBER_L0, '普卡'),
        (MemberLevel.MEMBER_L1, '银卡'),
        (MemberLevel.MEMBER_L2, '金卡'),
    )


class OutStorageConfigOrderAddressType(object):
    """ 发货地址配置 """
    # 仅专柜发货
    ONLY_SHOP = "only_shop"
    # 仅外仓发货
    ONLY_OUTSTORAGE = "only_outstorage"
    # 混合发货（既可专柜发货、又可外仓发货）
    MIX_STORAGE = "mix_storage"

    ORDER_ADDRESS_TYPE_CHOICES = (
        (ONLY_SHOP, '专柜'),
        (ONLY_OUTSTORAGE, '外仓'),
        (MIX_STORAGE, '混合仓库'),
    )


class OutStorageConfigAfsAddressType(object):
    """ 退货地址配置 """
    # 全退回专柜
    ALL_TO_SHOP = "all_to_shop"
    # 全退回外仓
    ALL_TO_OUTSTORAGE = "all_to_outstorage"
    # 哪里发货退哪里
    WHERE_TO_COME_AND_WHERE_TO_GO = "where_to_come_and_where_to_go"

    AFS_ADDRESS_TYPE_CHOICES = (
        (ALL_TO_SHOP, '退回专柜'),
        (ALL_TO_OUTSTORAGE, '退回外仓'),
        (WHERE_TO_COME_AND_WHERE_TO_GO, '退回原处'),
    )


class StepPayDeliverTimeType:
    ''' 预售商品发货时间配置 '''

    RELATIVE_TIME = "pay_relative_time"  # 相对发货时间  尾款支付成功后多久可以发货
    FIXED_TIME = "fixed_time"  # 绝对时间


# 欧莱雅集团可同步专柜列表
OLAY_SHOP_DICT_CONST = {
    "108030": "01",
    "108048": "45",
    "108063": "15",
    "108066": "83" 
}

class AddressGroupStatus(object):
    """
    可合-已齐、可合-未齐、无可合、已合单、在库已合、单单、无在库、其他
    """
    # 可合-已齐（多单在库、未合、无其他状态）
    MERGABLE_READY = "mergable_ready"
    # 可合-未齐（多单在库、未合、有其他状态）
    MERGABLE_WAITING = "mergable_waiting"
    # 已合单（多单在库、已合、无其他状态）
    MERGED = "merged"
    # 在库-已合（多单在库、已合、有其他状态）
    MERGED_IN_STORE = "merged_in_store"
    # 无可合（一单在库、有其他状态）
    UNMERGABLE = "unmergable"
    # 单单（一单在库、无其他状态）
    SINGLE = 'single'
    # 无在库（0单在库、有其他状态）
    NO_IN_STORE = 'no_in_store'
    # 其他
    OTHER = 'other'

    CHOICES = (
        (MERGABLE_READY, '可合-已齐'),
        (MERGABLE_WAITING, '可合-未齐'),
        (MERGED, "已合单"),
        (MERGED_IN_STORE, "在库-已合"),
        (UNMERGABLE, "无可合"),
        (SINGLE, "单单"),
        (NO_IN_STORE, "无在库"),
        (OTHER, "其他"),
    )
    # 存在未终结发货单的分组
    UNTERMINATE_STATUS_LIST = [MERGABLE_READY, MERGABLE_WAITING, MERGED, MERGED_IN_STORE, UNMERGABLE, NO_IN_STORE, SINGLE]

"""
发货单运送变更事件 枚举
合单、更新运单
TODO 修改地址、修改运输方式
"""
@unique
class DistributeShippingChangeEventEnum(Enum):

    merge_express_order = "merge_express_order"
    create_express_order = "create_express_order"
    update_shipping_number = "update_shipping_number"


    @classmethod
    def choices(cls):
        return (
            (cls.merge_express_order, "合单"),
            (cls.update_shipping_number, "更新运单"),
            (cls.create_express_order, "创建运单"),
        )

"""
拣货任务创建途径
"""
class MergePickPickTaskSource(object):
    # 揽月领任务
    LANYUE_ASSIGN_PICKTASK = 'lanyue_assign_picktask'
    # PDA扫发货单创建拣货任务
    PDA_SCAN_CREATE = 'pda_scan_create'

    CHOICES = (
        (LANYUE_ASSIGN_PICKTASK, '揽月领任务'),
        (PDA_SCAN_CREATE, 'PDA扫码创建任务'),
    )

# 合单拣货任务状态
class MergePickTaskStatus(object):
    # 已领取
    CLAIMED = "claimed"
    # 已完成
    COMPLETED = 'completed'
    # 已关闭
    CLOSED = 'closed'

    CHOICES = (
        (CLAIMED, '已领取'),
        (COMPLETED, '已完成'),
        (CLOSED, "已关闭"),
    )

    NAME_MAP = dict(CHOICES)

# 合单拣货任务覆盖 address group
class MergePickTaskCoverAddressGroup(object):
    # 部分
    PART = "part"
    # 全部
    ALL = 'all'

    CHOICES = (
        (PART, '部分'),
        (ALL, '全部'),
    )

    NAME_MAP = dict(CHOICES)


class RefundExchangeStatus(object):
    NONE = 'none'
    DOING = 'doing'
    DONE = 'done'

    REFUND_CHOICES = (
        (NONE, '无退货'),
        (DOING, '退货进行中'),
        (DONE, '退货完成'),
    )
    EXCHANGE_CHOICES = (
        (NONE, '无换货'),
        (DOING, '换货进行中'),
        (DONE, '换货完成'),
    )
    COMMON_DICT={
        "refund_status": dict(REFUND_CHOICES),
        "exchange_status": dict(EXCHANGE_CHOICES),
    }


from .constants import DocType


# 存放各文档的 mapping
MAPPINGS = {
    # 订单数据
    DocType.ORDER_V2: {
        "mapping": {
            "properties": {
                "id": {"type": "integer"},
                "order_number": {"type": "keyword"},
                "pay_status": {"type": "keyword"},
                "aftersale_status": {"type": "keyword"},
                "exchange_status": {"type": "keyword"},
                "assign_status": {"type": "keyword"},
                "shipping_status": {"type": "keyword"},
                "delivertime_status": {"type": "keyword"},
                "pay_method": {"type": "keyword"},
                "group_status": {"type": "keyword"},
                "user_id": {"type": "keyword"},
                "order_type": {"type": "keyword"},
                "sync_status": {"type": "boolean"},
                "sync_time": {"type": "date"},
                "updated_time": {"type": "date"},
                "created_time": {"type": "date"},
                "completed_time": {"type": "date"},
                "has_valid_to_assign": {"type": "boolean"},
                "has_valid_to_shipping": {"type": "boolean"},
                "lines": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "integer"},
                        "order_id": {"type": "integer"},
                        "brand_id": {"type": "integer"},
                        "shop_id": {"type": "integer"},
                        "shop_num": {"type": "keyword"},
                        "product_id": {"type": "integer"},
                        "productmain_id": {"type": "integer"},
                        "giftproduct_id": {"type": "integer"},
                        "shop_sku": {"type": "keyword"},
                        "hg_code": {"type": "keyword"},
                        "barcode": {"type": "keyword"},
                        "offer": {"type": "keyword"},
                        "if_gift": {"type": "boolean"},
                        "title": {
                            "type": "text",
                            "store": False,
                            "index_analyzer": "ik",
                            "search_analyzer": "ik"
                        }
                    }
                },
                "distributes": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "order_id": {"type": "integer"},
                        "order_number": {"type": "keyword"},
                        "status": {"type": "keyword"},
                        "number": {"type": "keyword"},
                        "created_time": {"type": "date"},
                        "completed_time": {"type": "date"}
                    }
                },
                "user": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "user_name": {"type": "keyword"},
                        "name": {"type": "keyword"}
                    }
                },
                "shipping_address": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "name": {"type": "keyword"},
                        "phone": {"type": "keyword"}
                    }
                }
            }
        }
    },
    DocType.ORDER_V3: {
        "mapping": {
            "properties": {
                "id": {"type": "integer"},
                "number": {"type": "keyword"},
                "pay_status": {"type": "keyword"},
                "aftersale_status": {"type": "keyword"},
                "exchange_status": {"type": "keyword"},
                "assign_status": {"type": "keyword"},
                "shipping_status": {"type": "keyword"},
                "delivertime_status": {"type": "keyword"},
                "pay_method": {"type": "keyword"},
                "group_status": {"type": "keyword"},
                "user_id": {"type": "keyword"},
                "order_type": {"type": "keyword"},
                "sync_status": {"type": "boolean"},
                "sync_time": {"type": "date"},
                "updated_time": {"type": "date"},
                "created_time": {"type": "date"},
                "completed_time": {"type": "date"},
                "has_valid_to_assign": {"type": "boolean"},
                "has_valid_to_shipping": {"type": "boolean"},
                "is_note": {"type": "boolean"},
                "is_operator_note": {"type": "boolean"},
                "is_single_product": {"type": "boolean"},
                "note": {"type": "text"},
                "operator_note": {"type": "text"},
                "actual_amount": {"type": "double"},
                "total_amount": {"type": "float"},
                "shipping_method": {"type": "keyword"},
                "lines": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "integer"},
                        "order_id": {"type": "integer"},
                        "brand_id": {"type": "integer"},
                        "shop_id": {"type": "integer"},
                        "shop_num": {"type": "keyword"},
                        "product_id": {"type": "integer"},
                        "productmain_id": {"type": "integer"},
                        "giftproduct_id": {"type": "integer"},
                        "shop_sku": {"type": "keyword"},
                        "hg_code": {"type": "keyword"},
                        "barcode": {"type": "keyword"},
                        "offer": {"type": "keyword"},
                        "if_gift": {"type": "boolean"},
                        "quantity": {"type": "integer"},
                        "title": {
                            "type": "text",
                            "store": False,
                            "index_analyzer": "ik",
                            "search_analyzer": "ik"
                        },
                        "sku_attr": {"type": "text"},
                        "main_image": {"type": "text"},
                    }
                },
                "distributes": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "order_id": {"type": "integer"},
                        "order_number": {"type": "keyword"},
                        "status": {"type": "keyword"},
                        "number": {"type": "keyword"},
                        "created_time": {"type": "date"},
                        "completed_time": {"type": "date"}
                    }
                },
                "user": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "user_name": {"type": "keyword"},
                        "name": {"type": "keyword"}
                    }
                },
                "shipping_address": {
                    "type": "nested",
                    "properties": {
                        "id": {"type": "keyword"},
                        "name": {"type": "keyword"},
                        "phone": {"type": "keyword"}
                    }
                }
            }
        }
    }
}



'''
定义 ES 各类 field, 实现逻辑关系行为
'''
from abc import ABC

from .constants import ESFieldType
from .exceptions import ESUtilParamException, ESUtilValueException, ESUtilTypeException
from .operators import FieldOperators


class ESField(ABC, FieldOperators):

    def __new__(cls, **kwargs):
        if 'type' not in kwargs:
            raise ESUtilParamException("缺少关键字参数 type")
        if 'path' not in kwargs:
            raise ESUtilParamException("缺少关键字参数 path")

        path = kwargs['path']
        type = kwargs['type']
        if '.' in path and type != ESFieldType.OBJECT:
            return object.__new__(NestedField)

        for sc in ESField.__subclasses__():
            if sc.type == type:
                return object.__new__(sc)
        else:
            raise ESUtilValueException(f"无法解析的字段类型: {type}")

    def __init__(self, *, path, type):
        self.path = path
        # self.type = type


class TextField(ESField):
    type = ESFieldType.TEXT


class DateTimeField(ESField):
    type = ESFieldType.DATETIME

    # TODO: 时间格式检查

    def __o__(self, other, op_str):
        return {"range": {self.path: {op_str: other, 'time_zone': '+08:00'}}}


class BooleanField(ESField):
    type = ESFieldType.BOOLEAN


class KeywordField(ESField):
    type = ESFieldType.KEYWORD


class NestedField(ESField):
    type = ESFieldType.NESTED

    def __init__(self, *, path: str, type: str):
        paths = path.split('.')
        self.path = paths[0]
        child_path = '.'.join(paths[1:])

        if type == ESFieldType.NESTED:
            type = ESFieldType.KEYWORD
        for sc in ESField.__subclasses__():
            if sc.type == type:
                self.child = sc(path=child_path, type=type)
                self.child.path = f'{self.path}.{self.child.path}'  # 初始化完, 补偿 path
                break
        else:
            raise ESUtilTypeException(f'未找到字段类型{type}')


class ObjectField(ESField):
    type = ESFieldType.OBJECT


class IntegerField(ESField):
    type = ESFieldType.INTEGER



"""
    服务依赖sparrow_core_go
"""

from sparrow_cloud.authorization.token import get_app_token, get_user_token

from .svc_config import SPARROW_CORE_GO_SERVICE
from .sparrow_api_path import CORE_GO_MEMBER_INFO_API_PATH
from .sparrow_api_path import CORE_GO_I_USER_INFO


from sparrow_cloud.restclient import rest_client

import logging
logger = logging.getLogger(__name__)


# core服务基类
class ServiceSparrowCoreGoBase(object):

    SERVICE_CONF = SPARROW_CORE_GO_SERVICE


class ServiceSparrowCoreGo(ServiceSparrowCoreGoBase):

    @classmethod
    def get_user_info_dict(cls, user_id):
        '''
        功能：根据user_id，获取会员信息
        参数：user_id
        返回：
            {
                "customer_user_id":customer_user_id,
                "customer_displayname":customer_name,
                "customer_username":customer_username,
                "customer_member_xx_level":customer_member_xx_level,
                "customer_member_number":customer_member_number,
                "user_id": customer_user_id,
                "username": customer_username,
                "user_displayname": customer_name + customer_username
            }

        '''
        if not user_id:
            return {}
        api_path = CORE_GO_MEMBER_INFO_API_PATH
        err_msg = ""
        try:
            token = get_user_token(user_id=user_id)
            logger.info(f"开始访问{api_path},参数分别为：SPARROW_CORE_GO_SERVICE={cls.SERVICE_CONF} user_id={user_id}")
            response = rest_client.get(SPARROW_CORE_GO_SERVICE, api_path=api_path, token=token)
            logger.info(f"访问结束：user_id={user_id} response={str(response)}")
            customer_displayname = ""
            customer_username = ""
            customer_member_xx_level = ""
            customer_member_number = ""
            if "name" in response:
                customer_displayname = response["name"]
            if "username" in response:
                customer_username = response["username"]
            if "level" in response:
                level_dict = response["level"]
                if "level" in level_dict:
                    customer_member_xx_level = level_dict["level"]
                if "number" in level_dict:
                    customer_member_number = level_dict["number"]

            customer_info_dict = {
                "customer_user_id": user_id,
                "customer_displayname": customer_displayname,
                "customer_username": customer_username,
                "customer_member_xx_level": customer_member_xx_level,
                "customer_member_number": customer_member_number
            }

            return customer_info_dict
        except BaseException as ex:
            err_msg = f"当远程调用{api_path}服务时，user_id={user_id}, 远程访问异常: {str(ex)}"
            logger.error(err_msg)
            return {}

    @classmethod
    def get_user_info_dict_all(cls, user_id):
        """
            由 user_id 获得 username  displayname
            只要数据库有, 一定会拿到, 不会屏蔽会员卡异常的情况

            {
                "customer_user_id": "c50a77b2dacc412bb9dbbe138453ff7c",
                "customer_displayname": "jxxxx",
                "customer_username": "1820xxxx05",
                "user_id": "c50a77b2dacc412bxxxxxx",
                "username": "182xxxxxx05",
                "user_displayname": "jxxxx 182xxxxxx05"
            }

        """
        if not user_id:
            return {}
        api_path = CORE_GO_I_USER_INFO.format(user_id)
        err_msg = ""
        try:
            logger.info(f"开始访问{api_path},参数分别为：SPARROW_CORE_GO_SERVICE={cls.SERVICE_CONF} user_id={user_id}")

            token = get_app_token()
            response = rest_client.get(SPARROW_CORE_GO_SERVICE, api_path=api_path, token=token)
            logger.info(f"访问结束：user_id={user_id} response={str(response)}")

            res_dict = response.get("data", {})

            customer_displayname = ""
            customer_username = ""

            customer_displayname = res_dict.get("name", customer_displayname)
            customer_username = res_dict.get("username", customer_username)

            if customer_displayname:
                user_displayname = customer_displayname + " " + customer_username
            else:
                user_displayname = customer_username

            customer_info_dict = {
                "customer_user_id": user_id,
                "customer_displayname": customer_displayname[:15],  # 用户昵称太长的话会引起数据错误, 所有表都是最长 15 个字符
                "customer_username": customer_username,
                "user_id": user_id,
                "username": customer_username,
                "user_displayname": user_displayname[:15],  # 用户昵称太长的话会引起数据错误, 所有表都是最长 15 个字符
            }

            return customer_info_dict
        except BaseException as ex:
            err_msg = f"当远程调用{api_path}服务时，user_id={user_id}, 远程访问异常: {str(ex)}"
            logger.error(err_msg)
            return {}



'''
ES 查询组件中要用的常量
'''


import os

from ...core.datastructures import ImmutablePropertyClassBase
from ...core.datastructures import ImmutableList


class ESParamOp(ImmutablePropertyClassBase):
    ''' 定义 ES 字段支持的运算 '''
    lt = 'lt'
    lte = 'lte'
    eq = 'eq'
    ne = 'ne'
    gt = 'gt'
    gte = 'gte'
    in_ = 'in_op'
    like = 'like_op'


class ESFieldType(ImmutablePropertyClassBase):
    ''' 定义 ES 字段类型 '''
    TEXT = 'text'
    KEYWORD = 'keyword'
    DATETIME = 'date'
    BOOLEAN = 'boolean'
    INTEGER = 'integer'

    NESTED = 'nested'
    OBJECT = 'object'

    SUB_OBJECT_FIELD = ImmutableList([NESTED, OBJECT])


class ESQueryGroupName(ImmutablePropertyClassBase):
    ''' ES 查询的 query_name '''
    MUST = 'must'
    MUST_NOT = 'must_not'
    SHOULD = 'should'
    AGGS = 'aggs'
    PAGINATION = 'pagination'
    SORT = 'sort'


# ES查询数据时默认每页数据大小
ES_QUERY_PAGE_SIZE_DEFAULT = int(os.environ.get('ES_QUERY_PAGE_SIZE_DEFAULT', '10'))
# ES查询数据最大条数, 超过这个数量就要使用 scroll 深度分页 查询
ES_MAX_RESULT_WINDOW = int(os.environ.get('ES_MAX_RESULT_WINDOW', '10000'))



from ..sparrow_shipping.constants import ShippingPartnerCodes
from ..core.constants import UserRole


BASE_INPARCEL_API_CODE = 20216000


class InParcelStatus(object):
    '''
    ### 换货单总单状态 ###
    '''
    INIT = "init"
    RECEIVED = "received"
    REFUSED = "refused"
    EXCEPTION = "exception"
    COMPLETE = "complete"

    INPARCEL_STATUS_CHOICES = (
        (INIT, "初始态"),
        (RECEIVED, "确收"),
        (REFUSED, "拒收"),
        (EXCEPTION, "异常"),
        (COMPLETE, "已拆"),
    )
    INPARCEL_STATUS_DICT = dict(INPARCEL_STATUS_CHOICES)


class InParcelSource(object):
    '''
    收包裹来源
    '''
    COMMON = "common"
    """ 普通收包裹（客人寄回、售后拦截、客人拒收等） """

    EXPRESS_RETURN = 'express_return'
    """ 快递公司退回包裹（比如因疫情原因，无法发往） """

    CHOICES = (
        (COMMON, "普通收包裹"),
        (EXPRESS_RETURN, "快递公司退回包裹"),
    )


class InParcelConnectCustomer(object):
    '''
    ### 联系顾客 ###
    '''
    YET_CONNECT = "yet_connect"
    NOT_CONNECT = "not_connect"
    CHOICES = (
        (YET_CONNECT, "已联系顾客"),
        (NOT_CONNECT, "未联系顾客"),
    )


class EventMapKey(object):

    TO_STATUS_KEY = "to_status"
    USER_ROLES_KEY = "user_roles"


class InParcelTaskType(object):
    '''
    ### 任务类型 ###
    '''
    # 确收
    RECEIVE = "receive"
    # 拒收
    REFUSE = "refuse"

    CHOICES = (
        (RECEIVE, "确收"),
        (REFUSE, "拒收"),
    )


class InParcelEventConst(object):
    '''
    ### 事件 ###
    '''
    # 确收
    RECEIVE = "receive"
    # 拒收
    REFUSE = "refuse"
    # 修改邮费支付方式
    UPDATE_POSTAGE_PAY_METHOD = 'update_postage_pay_method'
    # 删除
    DELETE_FROM_TASK = 'delete_from_task'
    # 自动关联商品
    AUTO_RELA_ENTITY = 'auto_rela_entity'
    # 手动增加关联商品
    MANUAL_ADD_RELA_ENTITY = 'manual_add_rela_entity'
    # 手动删除关联商品
    MANUAL_REMOVE_RELA_ENTITY = 'manual_remove_rela_entity'
    # 标记异常
    MARK_EXCEPTION = 'mark_exception'
    # 标记完成、拆包裹
    MARK_COMPLETE = 'mark_complete'

    CHOICES = (
        (RECEIVE, "确收"),
        (REFUSE, "拒收"),
        (UPDATE_POSTAGE_PAY_METHOD, "修改邮费支付方式"),
        (DELETE_FROM_TASK, "删除"),
        (AUTO_RELA_ENTITY, '自动关联商品'),
        (MANUAL_ADD_RELA_ENTITY, '手动增加关联商品'),
        (MANUAL_REMOVE_RELA_ENTITY, '手动删除关联商品'),
        (MARK_EXCEPTION, '标记异常'),
        (MARK_COMPLETE, '拆包裹(标记完成)'),
    )

    EVENT_2_NAME_MAP = {
        RECEIVE: "确收",
        REFUSE: "拒收",
        UPDATE_POSTAGE_PAY_METHOD: "修改邮费支付方式",
        DELETE_FROM_TASK: "删除",
        AUTO_RELA_ENTITY: '自动关联商品',
        MANUAL_ADD_RELA_ENTITY: '手动增加关联商品',
        MANUAL_REMOVE_RELA_ENTITY: '手动删除关联商品',
        MARK_EXCEPTION: '标记异常',
        MARK_COMPLETE: '拆包裹(标记完成)',
    }

    # 注意：
    # 1、当不存在EventMapKey.USER_ROLES_KEY的key时，说明所有角色都可以操作；
    # 2、当EventMapKey.TO_STATUS_KEY对应的value设置为None时，则说明动作不会造成状态变更；
    FROM_TO_MAP = {
        RECEIVE: {  # 收包裹
            InParcelStatus.INIT: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.RECEIVED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            InParcelStatus.EXCEPTION: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.RECEIVED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
        },
        REFUSE: {  # 拒收包裹
            InParcelStatus.INIT: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.REFUSED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            InParcelStatus.COMPLETE: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.REFUSED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        UPDATE_POSTAGE_PAY_METHOD: {  # 修改邮费支付方式
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        DELETE_FROM_TASK: {
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        MANUAL_ADD_RELA_ENTITY: {
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        MANUAL_REMOVE_RELA_ENTITY: {
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        MARK_EXCEPTION: {
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.EXCEPTION,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        MARK_COMPLETE: {
            InParcelStatus.RECEIVED: {
                EventMapKey.TO_STATUS_KEY: InParcelStatus.COMPLETE,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
    }


"""
收包裹的快递公司
京东
顺丰
中通
申通
韵达
邮政
圆通
德邦
其他
"""
in_parcel_express_partner_list = [
    ShippingPartnerCodes.JD,
    ShippingPartnerCodes.SF,
    ShippingPartnerCodes.ZTO,
    ShippingPartnerCodes.STO,
    ShippingPartnerCodes.YD,
    ShippingPartnerCodes.YZPY,
    ShippingPartnerCodes.YTO,
    ShippingPartnerCodes.DBL,
    ShippingPartnerCodes.OTHERS,
]


"""
收包裹关联类型

# 关联发货单、换货单、退货单
发货单Distribute ID models.CharField(max_length=100, primary_key=True, default=get_uuid4_hex, editable=False)
发货单Distribute number models.CharField("Distribute number", max_length=40, db_index=True, unique=True, default=get_sparrow_distribute_number)

换货单行ExchangeLine id = models.CharField(max_length=100, primary_key=True, default=get_uuid4_hex, editable=False)
换货单行ExchangeLine exline_number = models.CharField("换货分单编号", max_length=64, db_index=True, unique=True)

售后单lineID 售后单line = 
"""


class InParcelEntityRelaType:
    # 发货单的运单号
    DISTRIBUTE = 'distribute'
    # 退货单line
    AFTERSALE_LINE = 'aftersale_line'
    # 换货单line 客人寄回
    EXCHANGE_LINE_CUSTOMER = 'exchange_line_customer'
    # 换货单line 导购寄出
    EXCHANGE_LINE_GUIDE = 'exchange_line_guide'

    CHOICES = (
        (DISTRIBUTE, "发货单"),
        (AFTERSALE_LINE, "退货单line"),
        (EXCHANGE_LINE_CUSTOMER, "换货单line 客人寄回"),
        (EXCHANGE_LINE_GUIDE, "换货单line 导购寄出"),
    )
    # 给手动关联接口用的选项
    MANUAL_UPDATE_CHOICES = (
        ('refund', "退货单line"),
        ('exchange', "换货单line 客人寄回"),
    )
    MANUAL_UPDATE_CHOICES_MAP = {
        'refund': AFTERSALE_LINE,
        'exchange': EXCHANGE_LINE_CUSTOMER,
    }


"""
收包裹关联创建来源

收包裹时，客人回填时，客服修改售后运单号时


"""


class InParcelEntityRelaDataSource:
    # 收包裹
    RECEIVE_INPARCEL = 'receive_inparcel'
    # 客人回填运单号
    CUSTOMER_SUBMIT = 'customer_submit'
    # 客服修改运单号
    CUSSERVICE_UPDATE = 'cusservice_update'
    # 客服手动编辑关联商品
    CUSSERVICE_MANUAL_UPDATE = 'cusservice_manual_update'

    CHOICES = (
        (RECEIVE_INPARCEL, "收包裹时"),
        (CUSTOMER_SUBMIT, "客人回填运单号"),
        (CUSSERVICE_UPDATE, "客服修改售后运单号"),
        (CUSSERVICE_MANUAL_UPDATE, "客服手动编辑关联商品"),
    )

    USER_ROLE_TO_DATA_SOURCE_MAP = {
        UserRole.CUSTOMER: CUSTOMER_SUBMIT,
        UserRole.CUSSERVICE: CUSSERVICE_UPDATE,
    }



# -*- coding: utf-8 -*-

import time
from django.core.cache import caches
import logging
import hashlib

logger = logging.getLogger(__name__)


def timeit(f):

    def timed(*args, **kw):
        # import pdb; pdb.set_trace()
        ts = time.time()
        result = f(*args, **kw)
        te = time.time()
        # logger.info('func:%r args:[%r, %r] took: %2.4f sec' % (f.__name__, args, kw, te - ts))
        logger.info('==================func:%r  took: %2.4f sec' % (f.__name__, te - ts))
        return result
    return timed


def cache_func_result_decorator(ex=300):
    '''
    缓存某个方法的运行结果
    :params ex: 过期时间, 单位 s   默认 300
    '''
    def _decorator(func):
        def _wrapper(*args, **kwargs):
            cache_handler = caches.get('default')
            if not cache_handler:
                return func(*args, **kwargs)

            cache_key_info = str(func.__name__) + str(args) + str(kwargs)
            myMd5 = hashlib.md5()
            myMd5.update(cache_key_info)
            cache_key = myMd5.hexdigest()
            cached_data = cache_handler.get(cache_key)
            if cached_data:
                return cached_data['data']
            else:
                data = func(*args, **kwargs)
                cache_handler.set(cache_key, {'data': data}, ex)
                return data
        return _wrapper
    return _decorator



import unittest

from ...es_util.field import ESField
from ...es_util.constants import ESParamOp
from ...es_util.constants import ESFieldType


class TestOperators(unittest.TestCase):

    field = ESField(path="test_operator", type=ESFieldType.TEXT)

    test_value = 'test_value'

    integer_field = ESField(path='test_integer_operator', type=ESFieldType.INTEGER)

    test_integer_value = 2

    def test_term_operator(self):

        expected_result = {'term': {'test_operator': self.test_value}}

        self.assertDictEqual(self.field == self.test_value, expected_result, msg="Test Operator term failed")

    def test_terms_operator(self):

        expected_result = {'terms': {'test_operator': [self.test_value]}}

        self.assertDictEqual(self.field.in_([self.test_value]), expected_result, msg="Test Operator terms failed")

    def test_wildcard_operator(self):

        expected_result = {'wildcard': {'test_operator': self.test_value}}

        self.assertDictEqual(self.field.like(self.test_value), expected_result, msg="Test Operator wildcard failed")

    def test_range_operator(self):

        core = {ESParamOp.gt: self.test_integer_value}
        expected_result = {'range': {'test_integer_operator': core}}
        self.assertDictEqual(self.integer_field > self.test_integer_value, expected_result, "Test Operator range failed")

        core = {ESParamOp.gte: self.test_integer_value}
        expected_result = {'range': {'test_integer_operator': core}}
        self.assertDictEqual(self.integer_field >= self.test_integer_value, expected_result, "Test Operator range failed")

        core = {ESParamOp.lt: self.test_integer_value}
        expected_result = {'range': {'test_integer_operator': core}}
        self.assertDictEqual(self.integer_field < self.test_integer_value, expected_result, "Test Operator range failed")

        core = {ESParamOp.lte: self.test_integer_value}
        expected_result = {'range': {'test_integer_operator': core}}
        self.assertDictEqual(self.integer_field <= self.test_integer_value, expected_result, "Test Operator range failed")


if __name__ == '__main__':
    unittest.main()



import unittest

from ..es import ESDoor
from ..es.constants import DocType
from ..es.es_util.client import init_es


class TestOrderDocType(unittest.TestCase):

    def setUp(self):
        MockData().index_mock_data()

    def tearDown(self):
        MockData().delete_mock_data()

    def test_order_data(self):
        '''  '''
        door = ESDoor(
            DocType.ORDER,
            id=MockData.mock_order_id,
            phone=MockData.mock_user_username
        )
        res = door.main()
        print(res)


class MockData(object):

    mock_order_id = 0
    mock_distribute_id = '1783adaaa3ac4c618fe735006f32081d'
    mock_user_id = '1783a65aa3ac4c618fe735xx6f32081d'
    mock_user_name = '卡卡西'
    mock_user_username = '13512341234'

    mock_data = {
        'id': mock_order_id,
        'aftersale_status': 'done',
        'assign_status': 'completed',
        # 'completed_time': null
        'created_time': "2022-03-11T16:08:58.712557",
        # 'delivertime_status': null,
        'distributes': [
            {
                "created_time": "2022-03-11T16:09:11.004717",
                "completed": "2022-03-11T16:10:11.730537",
                "id": mock_distribute_id,
                "number": "220311160911-125738",
                "order_id": mock_order_id,
                "distribute_status": "haitao_return",
            }
        ],
        'has_valid_to_assign': 0,
        'has_valid_to_shipping': 0,
        "number": "22031199900000008",
        'order_type': "haitao",
        'pay_method': 'once',
        'pay_status': 'finished',
        "shipping_address": {
            "id": 0,
            "name": "mmm",
            "phone": mock_user_username,
        },
        "shipping_method": "express",
        "shipping_status": "init",
        "sync_status": 1,
        "sync_time": "2022-03-11T16:09:10.656545",
        "updated_time": "2022-03-11T16:09:12.120511",
        "user_id": mock_user_id,
        "user": {
            "id": mock_user_id,
            "name": mock_user_name,
            "user_name": mock_user_username,
        },
        'lines': [
            {
                "barcode": "858991004923",
                "brand_id": 2069,
                # "gift": null,
                # "gift_id": null,
                # "giftproduct_id": null,
                "hg_code": "18061010005",
                "id": 524046,
                "if_gift": 0,
                "offer": "1",
                "order_id": mock_order_id,
                "product_id": 482639,
                "productmain_id": "90824",
                "shop_id": 1158,
                "shop_num": "217002",
                "shop_sku": "858991004923",
                "title": "LED太阳光台灯无影手术灯反射技术无频闪学生台灯",
            }
        ]
    }

    def index_mock_data(self):
        ''' 测试之前添加一条假数据 '''
        es = init_es()
        es.index(index=DocType.ORDER, body=self.mock_data)

    def delete_mock_data(self):
        ''' 删除假数据 '''
        es = init_es()

        body = {
            'query': {
                'must': [
                    {
                        'term': {'id': self.mock_order_id}
                    },
                    {
                        'term': {'distributes.id': self.mock_distribute_id}
                    }
                ]
            }
        }

        es.delete(index=DocType.ORDER, body=body)




from .field import ESField
from .mock import ESMockDSLInterface
from .exceptions import ESUtilValueException, ESUtilTypeException, ESUtilParamException
from .operators import custom_op
from .constants import ESQueryGroupName, ESParamOp


class ESParam(ESMockDSLInterface):
    ''' 描述请求ES的参数 '''

    def __init__(self, field: ESField, op, other):
        self.field = field
        self.other = other
        self.op = op

        if isinstance(other, (list, tuple)):
            self.op = ESParamOp.in_

    def get_dsl(self):
        operator = custom_op(self.op)
        return operator(self.field, self.other)


class ESParamsForOneGroup(ESMockDSLInterface):
    ''' 一组同级别的查询条件, 各个查询条件之间的逻辑关系由上层 group_type 决定 '''

    def __init__(self, espms):
        self.espms = espms

    def get_dsl(self):
        terms = []
        for espm in self.espms:
            if issubclass(espm.__class__, ESMockDSLInterface):
                terms.append(espm.get_dsl())
            else:
                raise ESUtilTypeException(f'excepted {ESMockDSLInterface.__name__}, got {espm.__class__.__name__}')
        if len(terms) == 1:
            return terms[0]
        # TODO: 如何处理同一个 field 在 range 的合并
        return terms


class ESParamGroup(ESMockDSLInterface):
    ''' 将一组查询条件按照一定的逻辑组装在一起 '''
    __query_name = None

    def __new__(cls, *args, **kwargs):
        espm = args[0]
        query_name = kwargs.pop('query_name')

        # 参数是 ESParam 类型, 或者列表参数的每个元素都是 ESParam 或者参数类型是
        if issubclass(espm.__class__, ESMockDSLInterface):
            pass
        else:  # 参数类型有误
            raise ESUtilValueException("ES查询参数类型错误")

        for sub in cls.__subclasses__():
            if getattr(sub, "_" + sub.__name__ + "__query_name") == query_name:
                return object.__new__(sub)

        return super().__new__(cls)

    def __init__(self, espm: ESMockDSLInterface, *, query_name: str = None):

        self.espm = espm
        # self.query_name = query_name

    def get_dsl(self):
        return {self.query_name: self.espm.get_dsl()}

    def add_one_espm(self, es_param):
        if isinstance(self.espm, ESParam):
            self.espm = ESParamsForOneGroup([self.espm, es_param])
        elif isinstance(self.espm, ESParamsForOneGroup):
            self.espm.espms.append(es_param)
            # self.espm.append(es_param)
        else:
            raise ValueError(1)

    @property
    def query_name(self):
        return self.__query_name


class ESParamShouldGroup(ESParamGroup):
    ''' should query '''
    __query_name = ESQueryGroupName.SHOULD

    @property
    def query_name(self):
        return self.__query_name


class ESParamMustGroup(ESParamGroup):
    ''' must query '''
    __query_name = ESQueryGroupName.MUST

    @property
    def query_name(self):
        return self.__query_name


class ESParamMustNotGroup(ESParamGroup):
    '''' must_not query '''
    __query_name = ESQueryGroupName.MUST_NOT

    @property
    def query_name(self):
        return self.__query_name


class ESParamPageGroup(ESMockDSLInterface):
    ''' page Group '''
    __query_name = ESQueryGroupName.PAGINATION

    @property
    def query_name(self):
        return self.__query_name

    def __init__(self, page, page_size):
        self.page = page
        self.page_size = page_size

    def get_dsl(self):
        dsl = {
            'size': self.page_size,
            'from': (self.page - 1) * self.page_size
        }
        return dsl


class ESParamSortGroup(ESMockDSLInterface):
    __query_name = ESQueryGroupName.SORT

    @property
    def query_name(self):
        return self.__query_name

    def __init__(self, sort_string):
        '''
        :param sort_string: -created_time,updated_time desc 等
        '''

        self.__sort_string = sort_string

    def get_dsl(self):
        self.check()

        sort = {}
        for field in self.__sort_string.split(','):
            sort.update(self._sort_one_field(field))

        return {'sort': sort}

    def _sort_one_field(self, field_str: str):
        field = field_str
        sort = 'asc'
        if field_str.startswith('-'):
            field = field_str.strip('-')
            sort = 'desc'
        elif field_str.endswith(' desc'):
            field = field_str[:-5]
            sort = 'desc'
        elif field_str.endswith(' asc'):
            field = field_str[:-4]
            sort = 'asc'
        return {field: sort}

    def check(self):
        if not isinstance(self.__sort_string, str):
            raise ESUtilParamException(f'无效的排序参数: {self.__sort_string}')


class ESParamAggsGroup(ESMockDSLInterface):
    ''' aggs '''
    __query_name = ESQueryGroupName.AGGS

    @property
    def query_name(self):
        return self.__query_name
    # TODO: 实现
    pass



from ..core.constants import UserRole


class EventMapKey(object):

    TO_STATUS_KEY = "to_status"
    USER_ROLES_KEY = "user_roles"


class AfsLineStatus:
    ''' 退款单表体状态 '''
    # 待审核
    OPEN = "open"
    # 已拒绝
    REFUSED = "refused"
    # 已关闭
    CLOSED = 'closed'
    # 待客人寄回
    PENDING_CUSTOMER_RETURN = "pending_customer_return"
    # 客人已寄回
    CUSTOMER_RETURNED = "customer_returned"
    # 客服已收货
    CUSSERVICE_RECEIVED = "cusservice_received"
    # 进入退换货仓
    AFTERSALE_WH = "AftersaleWH"
    # 导购收货
    GUIDE_RECEIVED = "guide_received"
    # 待客服退货
    PENDING_CUSSERVICE_RETURN = "pending_cusservice_return"
    # 待外仓退货
    PENDING_OUTSTORAGE_RETURN = "pending_outstorage_return"
    # 待退款
    PENDING_REFUND = "pending_refund"
    # 已完成
    COMPLETED = 'completed'

    CHOICES = (
        # OPEN 待审核
        (OPEN, "待审核"),
        # 已拒绝
        (REFUSED, "已拒绝"),
        # 已关闭
        (CLOSED, '已关闭'),
        # 待客人寄回
        (PENDING_CUSTOMER_RETURN, "待客人寄回"),
        # 客人已寄回
        (CUSTOMER_RETURNED, "客人已寄回"),
        # 客服已收货
        (CUSSERVICE_RECEIVED, "客服已收货"),
        # 进入退换货仓
        (AFTERSALE_WH, "退货仓上架"),
        # 导购收货
        (GUIDE_RECEIVED, "导购收货"),
        # 待客服退货
        (PENDING_CUSSERVICE_RETURN, "待客服退货"),
        # 待外仓退货
        (PENDING_OUTSTORAGE_RETURN, "待外仓退货"),
        # 待退款
        (PENDING_REFUND, "待退款"),
        # 完成
        (COMPLETED, '已完成'),
    )

    CHOICES_FOR_A = CHOICES

    CHOICES_FOR_B = CHOICES_FOR_A

    CHOICES_FOR_P = CHOICES_FOR_A

    NAME_MAP_A = dict(CHOICES_FOR_A)


class AfsLineEventConst:
    '''
    ### 事件 ###
    '''

    # 拒绝退货申请
    REFUSE = 'refuse'
    # 取消退货
    CANCEL = "cancel"
    # 客户回填运单信息
    CUS_WRITE_BACK = "cus_write_back"
    # 客服收货
    CUSSERVICE_RECEIVE = 'cusservice_receive'
    # 取消客服收货
    CANCEL_CUSSERVICE_RECEIVE = 'cancel_cusservice_receive'
    # 售后仓入库
    ENTER_AFTERSALE_WH = 'enter_aftersale_wh'
    # 导购收货
    GUIDE_RECEIVE = "guide_receive"
    # 导购确退
    GUIDE_CONFIRM = 'guide_confirm'
    # 未出专柜审核，直接到待退款
    APPROVE_NO_ASSIGN = "approve_no_assign"
    # 售后完成
    COMPLETE = 'complete'

    EVENT_2_BUTTONNAME_MAP = {
        APPROVE_NO_ASSIGN: "未出专柜审核通过",
        CANCEL: "关闭",
        CUS_WRITE_BACK: "客户回填运单信息",
        GUIDE_RECEIVE: "导购收货",
        GUIDE_CONFIRM: "导购确退",
        ENTER_AFTERSALE_WH: "售后仓入库"
    }

    EVENT_2_CNAME_MAP = {
        APPROVE_NO_ASSIGN: "未出专柜审核通过",
        CANCEL: "关闭换货",
    }

    # 注意：
    # 1、当不存在EventMapKey.USER_ROLES_KEY的key时，说明所有角色都可以操作；
    # 2、当EventMapKey.TO_STATUS_KEY对应的value设置为None时，则说明动作不会造成状态变更；
    FROM_TO_MAP = {
        APPROVE_NO_ASSIGN: {  # 导购同意客户换货
            AfsLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.AUTO, UserRole.CUSSERVICE]
            }
        },
        REFUSE: {
            AfsLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.REFUSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.AUTO, UserRole.CUSSERVICE]
            }
        },
        CANCEL: {  # 取消换货
            AfsLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
            AfsLineStatus.PENDING_CUSTOMER_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
        },
        CUS_WRITE_BACK: {  # 客户回填运单信息
            AfsLineStatus.PENDING_CUSTOMER_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CUSTOMER_RETURNED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSTOMER, UserRole.CUSSERVICE]
            },
        },
        CUSSERVICE_RECEIVE: {
            AfsLineStatus.PENDING_CUSTOMER_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CUSSERVICE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            AfsLineStatus.CUSTOMER_RETURNED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CUSSERVICE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
        },
        CANCEL_CUSSERVICE_RECEIVE: {
            AfsLineStatus.CUSSERVICE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.CUSTOMER_RETURNED,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
        },
        ENTER_AFTERSALE_WH: {
            AfsLineStatus.CUSSERVICE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.AFTERSALE_WH,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
        },
        GUIDE_RECEIVE: {  # 打印
            AfsLineStatus.AFTERSALE_WH: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.GUIDE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            AfsLineStatus.PENDING_CUSSERVICE_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.GUIDE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
        },
        GUIDE_CONFIRM: {
            AfsLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.PENDING_CUSSERVICE_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.PENDING_CUSTOMER_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.CUSTOMER_RETURNED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.CUSSERVICE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.GUIDE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.AFTERSALE_WH: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
            AfsLineStatus.PENDING_OUTSTORAGE_RETURN: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.PENDING_REFUND,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            }
        },
        COMPLETE: {
            AfsLineStatus.PENDING_REFUND: {
                EventMapKey.TO_STATUS_KEY: AfsLineStatus.COMPLETED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.AUTO, UserRole.CUSTOMER]
            }
        }
    }

    # # 按钮排序
    # EVENT_CONST_SORT_MAP = {
    #     CANCEL: 0,
    #     COMMENT: 1,
    #     APPROVE: 2,
    #     CONFIRM: 2,
    #     GUIDE_EXPRESS: 2,
    #     PRINT: 3,
    #     CUS_SELF_SERVICE: 4,
    # }


class AfsLineGenerateFrom:
    SELECT = 'select'  # 创建售后时选中的
    VIRTUAL = 'virtual'  # 虚拟订单数据
    ORIGIN = 'origin'  # 创建售后前的订单(或虚拟订单)数据

    CHOICES = (
        (SELECT, "创建售后时选择的商品"),
        (VIRTUAL, "虚拟订单数据"),
        (ORIGIN, "创建售后前的订单(或虚拟订单)数据"),
    )

    NON_AFSLINE = [VIRTUAL, ORIGIN]



import unittest

from ...es_util.base import __EXAMPLE_DOC_TYPE as EXAMPLE_DOC_TYPE
from ...es_util.es_query_util import ESQueryUtil
from ...es_util.in_query_param import InQueryParam
from ...es_util.es_param import ESParamShouldGroup
from ...es_util.es_param import ESParamPageGroup


class TestESQueryUtil(unittest.TestCase):

    example_util = ESQueryUtil(EXAMPLE_DOC_TYPE)

    def test_properties(self):
        ''' 测试 ESQueryUtil 对不可变属性的控制 '''

        self.assertRaises(
            AttributeError,
            setattr,
            self.example_util,
            'doc_type',
            'test_doc_type'
        )

        self.assertRaises(
            AttributeError,
            setattr,
            self.example_util,
            'mapping',
            {}
        )

        self.assertRaises(
            AttributeError,
            setattr,
            self.example_util,
            'query_mapping',
            {}
        )

    def test_get_es_query_param(self):
        in_param = InQueryParam(in_param_key='all_number', in_param_value='18512341234')
        res = self.example_util.get_es_query_param(in_param)

        self.assertIsInstance(res, ESParamShouldGroup, msg="Test ESQueryUtil.get_es_query_param Failed")

        expected_result = {
            'should': [
                {
                    'term': {'number': '18512341234'},
                },
                {
                    'nested': {
                        'path': 'inner',
                        'query': {
                            'bool': {
                                'filter': {
                                    'term': {
                                        'inner.number': '18512341234'
                                    }
                                }
                            }
                        }
                    }
                }
            ]
        }

        self.assertDictEqual(res.get_dsl(), expected_result, msg="Test ESQueryUtil.get_es_query_param Failed")

    def test_get_page_param_group(self):
        res = self.example_util.get_page_param_group(page=2, page_size=10)

        self.assertIsInstance(res, ESParamPageGroup)

        expected_result = {'from': 10, 'size': 10}

        self.assertDictEqual(res.get_dsl(), expected_result, msg="Test ESQueryUtil.get_page_param Failed")


if __name__ == '__main__':
    unittest.main()



import unittest

from ...es_util.base import _doc_types_conf
from ...es_util.base import __EXAMPLE_DOC_TYPE as EXAMPLE_DOC_TYPE
from ...es_util.base import __EXAMPLE_MAPPING as EXAMPLE_MAPPING
from ...es_util.base import _doc_types_conf_mapping_key
from ...es_util.base import _doc_types_conf_query_mapping_key
from ...es_util.constants import ESFieldType


class TestConfig(unittest.TestCase):

    def test_assert_doc_type_in_conf(self):
        ''' 判断示例文档类型注册成功 '''

        expr = EXAMPLE_DOC_TYPE in _doc_types_conf
        self.assertTrue(expr=expr, msg="测试文档类型未加载到配置中")

    def test_assert_all_fields_in_conf(self):
        ''' 示例文档类型注册后配置正确 '''
        query_conf = _doc_types_conf[EXAMPLE_DOC_TYPE][_doc_types_conf_query_mapping_key]
        mapping_conf = _doc_types_conf[EXAMPLE_DOC_TYPE][_doc_types_conf_mapping_key]['mapping']['properties']
        for field in EXAMPLE_MAPPING['mapping']['properties']:
            if EXAMPLE_MAPPING['mapping']['properties'][field]['type'] not in ESFieldType.SUB_OBJECT_FIELD:
                # 所有 field 都在配置中
                expr = field in _doc_types_conf[EXAMPLE_DOC_TYPE][_doc_types_conf_query_mapping_key]
                self.assertTrue(expr=expr, msg=f"示例文档类型配置错误, {field}不在配置中")
                # 所有 field 的配置类型都相同
                expr2 = query_conf[field]['type'] == mapping_conf[field]['type']
                self.assertTrue(expr=expr2, msg=f"示例文档类型 {field} 在 mapping 和 query_mapping 中的类型不一致.")


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase
import unittest

from ...es_util.field import ESField
from ...es_util.field import TextField
from ...es_util.field import DateTimeField
from ...es_util.field import BooleanField
from ...es_util.field import KeywordField
from ...es_util.field import NestedField
from ...es_util.field import ObjectField
from ...es_util.field import IntegerField
from ...es_util.constants import ESFieldType


class TestESField(TestCase):

    def test_text_field(self):
        text_field = ESField(path='test_text_field_path_path', type=ESFieldType.TEXT)

        self.assertIsInstance(text_field, cls=TextField, msg="初始化 TextField 失败")

    def test_datetime_field(self):
        datetime_field = ESField(path='test_datetime_field_path', type=ESFieldType.DATETIME)

        self.assertIsInstance(datetime_field, DateTimeField, msg="初始化 DateTimeField 失败")

    def test_boolean_field(self):
        boolean_field = ESField(path='test_boolean_field_path', type=ESFieldType.BOOLEAN)

        self.assertIsInstance(boolean_field, BooleanField, msg="初始化 BooleanField 失败")

    def test_keyword_field(self):
        keyword_field = ESField(path='test_keyword_field_path', type=ESFieldType.KEYWORD)

        self.assertIsInstance(keyword_field, KeywordField, msg="初始化 KeywordField 失败")

    def test_nested_field(self):
        ''' nested 是指第一层 path 的类型, 传入的 type 是指目标查询字段类型 '''
        nested_field = ESField(path='test_nested_field.child_path', type=ESFieldType.KEYWORD)

        self.assertIsInstance(nested_field, NestedField, msg="初始化 NestedField 失败")

    def test_object_field(self):
        object_field = ESField(path='test_nested_field.child_path', type=ESFieldType.OBJECT)

        self.assertIsInstance(object_field, ObjectField, msg="初始化 ObjectField 失败")

    def test_integer_field(self):
        integer_field = ESField(path='test_integer_field_path', type=ESFieldType.INTEGER)

        self.assertIsInstance(integer_field, IntegerField, msg="初始化 IntegerField 失败")


if __name__ == '__main__':
    unittest.main()



import unittest
from .test_base import TestConfig
from .test_client import TestClient
from .test_field import TestESField
from .test_es_door import TestESDoor
from .test_es_param import TestESParam
from .test_operators import TestOperators
from .test_es_builder import TestESBuilder
from .test_mock import TestESMockDSLInterface
from .test_es_query_util import TestESQueryUtil
from .test_in_query_param import TestInQueryParam


def whole_suite():
    # 创建测试加载器
    loader = unittest.TestLoader()
    # 创建测试包
    suite = unittest.TestSuite()
    # 遍历所有测试类

    test_cases = [
        TestConfig,
        TestESMockDSLInterface,
        TestOperators,
        TestESField,
        TestInQueryParam,
        TestESParam,
        TestESQueryUtil,
        TestClient,
        TestESBuilder,
        TestESDoor,
    ]

    for test_class in test_cases:
        # 从测试类中加载测试用例
        tests = loader.loadTestsFromTestCase(test_class)
        # 将测试用例添加到测试包中
        suite.addTests(tests)
    return suite


def test_es_util():
    ''' 运行所有的ES组件测试用例 '''
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(whole_suite())


if __name__ == '__main__':
    test_es_util()
